# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class PanelSchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'db': 'str',
        'decimals': 'str',
        'panel_data': 'list[PanelDataSchema]',
        'panel_name': 'str',
        'panel_query': 'PanelSchemaPanelQuery',
        'panel_type': 'PanelSchemaPanelType',
        'panel_uid': 'str',
        'time_range': 'PanelSchemaTimeRange',
        'unit_type': 'PanelSchemaUnitType',
        'y_label': 'str',
        'y_max': 'str',
        'y_min': 'str'
    }

    attribute_map = {
        'db': 'db',
        'decimals': 'decimals',
        'panel_data': 'panelData',
        'panel_name': 'panelName',
        'panel_query': 'panelQuery',
        'panel_type': 'panelType',
        'panel_uid': 'panelUid',
        'time_range': 'timeRange',
        'unit_type': 'unitType',
        'y_label': 'yLabel',
        'y_max': 'yMax',
        'y_min': 'yMin'
    }

    def __init__(self, db=None, decimals=None, panel_data=None, panel_name=None, panel_query=None, panel_type=None, panel_uid=None, time_range=None, unit_type=None, y_label=None, y_max=None, y_min=None):  # noqa: E501
        """PanelSchema - a model defined in Swagger"""  # noqa: E501

        self._db = None
        self._decimals = None
        self._panel_data = None
        self._panel_name = None
        self._panel_query = None
        self._panel_type = None
        self._panel_uid = None
        self._time_range = None
        self._unit_type = None
        self._y_label = None
        self._y_max = None
        self._y_min = None
        self.discriminator = None

        if db is not None:
            self.db = db
        if decimals is not None:
            self.decimals = decimals
        if panel_data is not None:
            self.panel_data = panel_data
        if panel_name is not None:
            self.panel_name = panel_name
        if panel_query is not None:
            self.panel_query = panel_query
        if panel_type is not None:
            self.panel_type = panel_type
        if panel_uid is not None:
            self.panel_uid = panel_uid
        if time_range is not None:
            self.time_range = time_range
        if unit_type is not None:
            self.unit_type = unit_type
        if y_label is not None:
            self.y_label = y_label
        if y_max is not None:
            self.y_max = y_max
        if y_min is not None:
            self.y_min = y_min

    @property
    def db(self):
        """Gets the db of this PanelSchema.  # noqa: E501

        database name  # noqa: E501

        :return: The db of this PanelSchema.  # noqa: E501
        :rtype: str
        """
        return self._db

    @db.setter
    def db(self, db):
        """Sets the db of this PanelSchema.

        database name  # noqa: E501

        :param db: The db of this PanelSchema.  # noqa: E501
        :type: str
        """

        self._db = db

    @property
    def decimals(self):
        """Gets the decimals of this PanelSchema.  # noqa: E501

        decimal value  # noqa: E501

        :return: The decimals of this PanelSchema.  # noqa: E501
        :rtype: str
        """
        return self._decimals

    @decimals.setter
    def decimals(self, decimals):
        """Sets the decimals of this PanelSchema.

        decimal value  # noqa: E501

        :param decimals: The decimals of this PanelSchema.  # noqa: E501
        :type: str
        """

        self._decimals = decimals

    @property
    def panel_data(self):
        """Gets the panel_data of this PanelSchema.  # noqa: E501

        Panel data  # noqa: E501

        :return: The panel_data of this PanelSchema.  # noqa: E501
        :rtype: list[PanelDataSchema]
        """
        return self._panel_data

    @panel_data.setter
    def panel_data(self, panel_data):
        """Sets the panel_data of this PanelSchema.

        Panel data  # noqa: E501

        :param panel_data: The panel_data of this PanelSchema.  # noqa: E501
        :type: list[PanelDataSchema]
        """

        self._panel_data = panel_data

    @property
    def panel_name(self):
        """Gets the panel_name of this PanelSchema.  # noqa: E501

        Name of the panel  # noqa: E501

        :return: The panel_name of this PanelSchema.  # noqa: E501
        :rtype: str
        """
        return self._panel_name

    @panel_name.setter
    def panel_name(self, panel_name):
        """Sets the panel_name of this PanelSchema.

        Name of the panel  # noqa: E501

        :param panel_name: The panel_name of this PanelSchema.  # noqa: E501
        :type: str
        """

        self._panel_name = panel_name

    @property
    def panel_query(self):
        """Gets the panel_query of this PanelSchema.  # noqa: E501


        :return: The panel_query of this PanelSchema.  # noqa: E501
        :rtype: PanelSchemaPanelQuery
        """
        return self._panel_query

    @panel_query.setter
    def panel_query(self, panel_query):
        """Sets the panel_query of this PanelSchema.


        :param panel_query: The panel_query of this PanelSchema.  # noqa: E501
        :type: PanelSchemaPanelQuery
        """

        self._panel_query = panel_query

    @property
    def panel_type(self):
        """Gets the panel_type of this PanelSchema.  # noqa: E501


        :return: The panel_type of this PanelSchema.  # noqa: E501
        :rtype: PanelSchemaPanelType
        """
        return self._panel_type

    @panel_type.setter
    def panel_type(self, panel_type):
        """Sets the panel_type of this PanelSchema.


        :param panel_type: The panel_type of this PanelSchema.  # noqa: E501
        :type: PanelSchemaPanelType
        """

        self._panel_type = panel_type

    @property
    def panel_uid(self):
        """Gets the panel_uid of this PanelSchema.  # noqa: E501

        Unique identifier for panel  # noqa: E501

        :return: The panel_uid of this PanelSchema.  # noqa: E501
        :rtype: str
        """
        return self._panel_uid

    @panel_uid.setter
    def panel_uid(self, panel_uid):
        """Sets the panel_uid of this PanelSchema.

        Unique identifier for panel  # noqa: E501

        :param panel_uid: The panel_uid of this PanelSchema.  # noqa: E501
        :type: str
        """

        self._panel_uid = panel_uid

    @property
    def time_range(self):
        """Gets the time_range of this PanelSchema.  # noqa: E501


        :return: The time_range of this PanelSchema.  # noqa: E501
        :rtype: PanelSchemaTimeRange
        """
        return self._time_range

    @time_range.setter
    def time_range(self, time_range):
        """Sets the time_range of this PanelSchema.


        :param time_range: The time_range of this PanelSchema.  # noqa: E501
        :type: PanelSchemaTimeRange
        """

        self._time_range = time_range

    @property
    def unit_type(self):
        """Gets the unit_type of this PanelSchema.  # noqa: E501


        :return: The unit_type of this PanelSchema.  # noqa: E501
        :rtype: PanelSchemaUnitType
        """
        return self._unit_type

    @unit_type.setter
    def unit_type(self, unit_type):
        """Sets the unit_type of this PanelSchema.


        :param unit_type: The unit_type of this PanelSchema.  # noqa: E501
        :type: PanelSchemaUnitType
        """

        self._unit_type = unit_type

    @property
    def y_label(self):
        """Gets the y_label of this PanelSchema.  # noqa: E501

        yLabel values  # noqa: E501

        :return: The y_label of this PanelSchema.  # noqa: E501
        :rtype: str
        """
        return self._y_label

    @y_label.setter
    def y_label(self, y_label):
        """Sets the y_label of this PanelSchema.

        yLabel values  # noqa: E501

        :param y_label: The y_label of this PanelSchema.  # noqa: E501
        :type: str
        """

        self._y_label = y_label

    @property
    def y_max(self):
        """Gets the y_max of this PanelSchema.  # noqa: E501

        yMax value  # noqa: E501

        :return: The y_max of this PanelSchema.  # noqa: E501
        :rtype: str
        """
        return self._y_max

    @y_max.setter
    def y_max(self, y_max):
        """Sets the y_max of this PanelSchema.

        yMax value  # noqa: E501

        :param y_max: The y_max of this PanelSchema.  # noqa: E501
        :type: str
        """

        self._y_max = y_max

    @property
    def y_min(self):
        """Gets the y_min of this PanelSchema.  # noqa: E501

        yMin value  # noqa: E501

        :return: The y_min of this PanelSchema.  # noqa: E501
        :rtype: str
        """
        return self._y_min

    @y_min.setter
    def y_min(self, y_min):
        """Sets the y_min of this PanelSchema.

        yMin value  # noqa: E501

        :param y_min: The y_min of this PanelSchema.  # noqa: E501
        :type: str
        """

        self._y_min = y_min

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PanelSchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PanelSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class PlaybookSchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'description': 'str',
        'playbook_name': 'str',
        'rules': 'list[str]',
        'synopsis': 'str',
        'classification': 'str'
    }

    attribute_map = {
        'description': 'description',
        'playbook_name': 'playbook-name',
        'rules': 'rules',
        'synopsis': 'synopsis',
        'classification': 'classification'
    }

    def __init__(self, description=None, playbook_name=None, rules=None, synopsis=None, classification=None):  # noqa: E501
        """PlaybookSchema - a model defined in Swagger"""  # noqa: E501

        self._description = None
        self._playbook_name = None
        self._rules = None
        self._synopsis = None
        self._classification = None
        self.discriminator = None

        if description is not None:
            self.description = description
        self.playbook_name = playbook_name
        if rules is not None:
            self.rules = rules
        if synopsis is not None:
            self.synopsis = synopsis
        if classification is not None:
            self.classification = classification

    @property
    def description(self):
        """Gets the description of this PlaybookSchema.  # noqa: E501

        Description about this playbook  # noqa: E501

        :return: The description of this PlaybookSchema.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this PlaybookSchema.

        Description about this playbook  # noqa: E501

        :param description: The description of this PlaybookSchema.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def playbook_name(self):
        """Gets the playbook_name of this PlaybookSchema.  # noqa: E501

        Name of the playbook. Should be of pattern [a-zA-Z][a-zA-Z0-9_-]*  # noqa: E501

        :return: The playbook_name of this PlaybookSchema.  # noqa: E501
        :rtype: str
        """
        return self._playbook_name

    @playbook_name.setter
    def playbook_name(self, playbook_name):
        """Sets the playbook_name of this PlaybookSchema.

        Name of the playbook. Should be of pattern [a-zA-Z][a-zA-Z0-9_-]*  # noqa: E501

        :param playbook_name: The playbook_name of this PlaybookSchema.  # noqa: E501
        :type: str
        """
        if playbook_name is None:
            raise ValueError("Invalid value for `playbook_name`, must not be `None`")  # noqa: E501
        if playbook_name is not None and len(playbook_name) > 64:
            raise ValueError("Invalid value for `playbook_name`, length must be less than or equal to `64`")  # noqa: E501
        if playbook_name is not None and not re.search(r'^[a-zA-Z][a-zA-Z0-9_-]*$', playbook_name):  # noqa: E501
            raise ValueError(r"Invalid value for `playbook_name`, must be a follow pattern or equal to `/^[a-zA-Z][a-zA-Z0-9_-]*$/`")  # noqa: E501

        self._playbook_name = playbook_name

    @property
    def rules(self):
        """Gets the rules of this PlaybookSchema.  # noqa: E501


        :return: The rules of this PlaybookSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._rules

    @rules.setter
    def rules(self, rules):
        """Sets the rules of this PlaybookSchema.


        :param rules: The rules of this PlaybookSchema.  # noqa: E501
        :type: list[str]
        """

        self._rules = rules

    @property
    def synopsis(self):
        """Gets the synopsis of this PlaybookSchema.  # noqa: E501

        Short description about this playbook  # noqa: E501

        :return: The synopsis of this PlaybookSchema.  # noqa: E501
        :rtype: str
        """
        return self._synopsis

    @synopsis.setter
    def synopsis(self, synopsis):
        """Sets the synopsis of this PlaybookSchema.

        Short description about this playbook  # noqa: E501

        :param synopsis: The synopsis of this PlaybookSchema.  # noqa: E501
        :type: str
        """

        self._synopsis = synopsis

    @property
    def classification(self):
        """Gets the classification of this PlaybookSchema.  # noqa: E501

        Classification info for this playbook  # noqa: E501

        :return: The classification of this PlaybookSchema.  # noqa: E501
        :rtype: str
        """
        return self._classification

    @classification.setter
    def classification(self, classification):
        """Sets the classification of this PlaybookSchema.

        Classification info for this playbook  # noqa: E501

        :param classification: The classification of this PlaybookSchema.  # noqa: E501
        :type: str
        """

        self._classification = classification

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PlaybookSchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PlaybookSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class RuleSchemaRuleproperties(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'author': 'str',
        'catalogue': 'RuleSchemaRulepropertiesCatalogue',
        'contributor': 'str',
        '_date': 'str',
        'helper_files': 'list[RuleSchemaRulepropertiesHelperfiles]',
        'supported_devices': 'RuleSchemaRulepropertiesSupporteddevices',
        'supported_healthbot_version': 'str',
        'version': 'int',
        'apply_macro': 'list[ApplyMacroSchema]'
    }

    attribute_map = {
        'author': 'author',
        'catalogue': 'catalogue',
        'contributor': 'contributor',
        '_date': 'date',
        'helper_files': 'helper-files',
        'supported_devices': 'supported-devices',
        'supported_healthbot_version': 'supported-healthbot-version',
        'version': 'version',
        'apply_macro': 'apply-macro'
    }

    def __init__(self, author=None, catalogue=None, contributor=None, _date=None, helper_files=None, supported_devices=None, supported_healthbot_version=None, version=None, apply_macro=None):  # noqa: E501
        """RuleSchemaRuleproperties - a model defined in Swagger"""  # noqa: E501

        self._author = None
        self._catalogue = None
        self._contributor = None
        self.__date = None
        self._helper_files = None
        self._supported_devices = None
        self._supported_healthbot_version = None
        self._version = None
        self._apply_macro = None
        self.discriminator = None

        if author is not None:
            self.author = author
        if catalogue is not None:
            self.catalogue = catalogue
        if contributor is not None:
            self.contributor = contributor
        if _date is not None:
            self._date = _date
        if helper_files is not None:
            self.helper_files = helper_files
        if supported_devices is not None:
            self.supported_devices = supported_devices
        if supported_healthbot_version is not None:
            self.supported_healthbot_version = supported_healthbot_version
        if version is not None:
            self.version = version
        if apply_macro is not None:
            self.apply_macro = apply_macro

    @property
    def author(self):
        """Gets the author of this RuleSchemaRuleproperties.  # noqa: E501

        E-mail address of the rule writer  # noqa: E501

        :return: The author of this RuleSchemaRuleproperties.  # noqa: E501
        :rtype: str
        """
        return self._author

    @author.setter
    def author(self, author):
        """Sets the author of this RuleSchemaRuleproperties.

        E-mail address of the rule writer  # noqa: E501

        :param author: The author of this RuleSchemaRuleproperties.  # noqa: E501
        :type: str
        """
        if author is not None and not re.search(r'^([\\w\\-_\\.]+)@([\\w])+[.][\\w]+$', author):  # noqa: E501
            raise ValueError(r"Invalid value for `author`, must be a follow pattern or equal to `/^([\\w\\-_\\.]+)@([\\w])+[.][\\w]+$/`")  # noqa: E501

        self._author = author

    @property
    def catalogue(self):
        """Gets the catalogue of this RuleSchemaRuleproperties.  # noqa: E501


        :return: The catalogue of this RuleSchemaRuleproperties.  # noqa: E501
        :rtype: RuleSchemaRulepropertiesCatalogue
        """
        return self._catalogue

    @catalogue.setter
    def catalogue(self, catalogue):
        """Sets the catalogue of this RuleSchemaRuleproperties.


        :param catalogue: The catalogue of this RuleSchemaRuleproperties.  # noqa: E501
        :type: RuleSchemaRulepropertiesCatalogue
        """

        self._catalogue = catalogue

    @property
    def contributor(self):
        """Gets the contributor of this RuleSchemaRuleproperties.  # noqa: E501


        :return: The contributor of this RuleSchemaRuleproperties.  # noqa: E501
        :rtype: str
        """
        return self._contributor

    @contributor.setter
    def contributor(self, contributor):
        """Sets the contributor of this RuleSchemaRuleproperties.


        :param contributor: The contributor of this RuleSchemaRuleproperties.  # noqa: E501
        :type: str
        """
        allowed_values = ["juniper", "external"]  # noqa: E501
        if contributor not in allowed_values:
            raise ValueError(
                "Invalid value for `contributor` ({0}), must be one of {1}"  # noqa: E501
                .format(contributor, allowed_values)
            )

        self._contributor = contributor

    @property
    def _date(self):
        """Gets the _date of this RuleSchemaRuleproperties.  # noqa: E501


        :return: The _date of this RuleSchemaRuleproperties.  # noqa: E501
        :rtype: str
        """
        return self.__date

    @_date.setter
    def _date(self, _date):
        """Sets the _date of this RuleSchemaRuleproperties.


        :param _date: The _date of this RuleSchemaRuleproperties.  # noqa: E501
        :type: str
        """
        if _date is not None and not re.search(r'^(19|20)\\d\\d[\/](0[1-9]|1[012])[\/](0[1-9]|[12][0-9]|3[01])$', _date):  # noqa: E501
            raise ValueError(r"Invalid value for `_date`, must be a follow pattern or equal to `/^(19|20)\\d\\d[\/](0[1-9]|1[012])[\/](0[1-9]|[12][0-9]|3[01])$/`")  # noqa: E501

        self.__date = _date

    @property
    def helper_files(self):
        """Gets the helper_files of this RuleSchemaRuleproperties.  # noqa: E501


        :return: The helper_files of this RuleSchemaRuleproperties.  # noqa: E501
        :rtype: list[RuleSchemaRulepropertiesHelperfiles]
        """
        return self._helper_files

    @helper_files.setter
    def helper_files(self, helper_files):
        """Sets the helper_files of this RuleSchemaRuleproperties.


        :param helper_files: The helper_files of this RuleSchemaRuleproperties.  # noqa: E501
        :type: list[RuleSchemaRulepropertiesHelperfiles]
        """

        self._helper_files = helper_files

    @property
    def supported_devices(self):
        """Gets the supported_devices of this RuleSchemaRuleproperties.  # noqa: E501


        :return: The supported_devices of this RuleSchemaRuleproperties.  # noqa: E501
        :rtype: RuleSchemaRulepropertiesSupporteddevices
        """
        return self._supported_devices

    @supported_devices.setter
    def supported_devices(self, supported_devices):
        """Sets the supported_devices of this RuleSchemaRuleproperties.


        :param supported_devices: The supported_devices of this RuleSchemaRuleproperties.  # noqa: E501
        :type: RuleSchemaRulepropertiesSupporteddevices
        """

        self._supported_devices = supported_devices

    @property
    def supported_healthbot_version(self):
        """Gets the supported_healthbot_version of this RuleSchemaRuleproperties.  # noqa: E501

        Healthbot version in which is rule is supported  # noqa: E501

        :return: The supported_healthbot_version of this RuleSchemaRuleproperties.  # noqa: E501
        :rtype: str
        """
        return self._supported_healthbot_version

    @supported_healthbot_version.setter
    def supported_healthbot_version(self, supported_healthbot_version):
        """Sets the supported_healthbot_version of this RuleSchemaRuleproperties.

        Healthbot version in which is rule is supported  # noqa: E501

        :param supported_healthbot_version: The supported_healthbot_version of this RuleSchemaRuleproperties.  # noqa: E501
        :type: str
        """
        if supported_healthbot_version is not None and not re.search(r'^(\\d){1,2}[-.](\\d){1,2}[-.](\\d){1,2}$', supported_healthbot_version):  # noqa: E501
            raise ValueError(r"Invalid value for `supported_healthbot_version`, must be a follow pattern or equal to `/^(\\d){1,2}[-.](\\d){1,2}[-.](\\d){1,2}$/`")  # noqa: E501

        self._supported_healthbot_version = supported_healthbot_version

    @property
    def version(self):
        """Gets the version of this RuleSchemaRuleproperties.  # noqa: E501

        Rule version, an integer value needs to be incremented for any major change  # noqa: E501

        :return: The version of this RuleSchemaRuleproperties.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this RuleSchemaRuleproperties.

        Rule version, an integer value needs to be incremented for any major change  # noqa: E501

        :param version: The version of this RuleSchemaRuleproperties.  # noqa: E501
        :type: int
        """

        self._version = version

    @property
    def apply_macro(self):
        """Gets the apply_macro of this RuleSchemaRuleproperties.  # noqa: E501


        :return: The apply_macro of this RuleSchemaRuleproperties.  # noqa: E501
        :rtype: list[ApplyMacroSchema]
        """
        return self._apply_macro

    @apply_macro.setter
    def apply_macro(self, apply_macro):
        """Sets the apply_macro of this RuleSchemaRuleproperties.


        :param apply_macro: The apply_macro of this RuleSchemaRuleproperties.  # noqa: E501
        :type: list[ApplyMacroSchema]
        """

        self._apply_macro = apply_macro

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RuleSchemaRuleproperties, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RuleSchemaRuleproperties):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



class BaseModule(object):
    def __init__(self, hbot, url=None):
        """
        :param object hbot: :class:`jnpr.healthbot.HealthBotClient` client instance
        :param str url: either hb.url or hb.config_url. default is hb.url
        """

        self.hbot = hbot
        if url is None:
            self.url = hbot.url
        else:
            self.url = url

    @property
    def api(self):
        return self.hbot.hbot_session

    @property
    def api_client(self):
        return self.hbot.api_client

    @property
    def authorization(self):
        return 'x-iam-token ' + self.hbot.user_token.access_token



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from jnpr.healthbot.swagger.api_client import ApiClient


class FactsApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def retrieve_iceberg_device_device_facts_by_id(self, device_id, **kwargs):  # noqa: E501
        """Get a device's facts.  # noqa: E501

        Get the fact details of a device by its `device-id`.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.retrieve_iceberg_device_device_facts_by_id(device_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str device_id: ID of device-id (required)
        :param str x_iam_token: authentication header object
        :param bool working: true queries un-committed configuration
        :param bool update: true will first update facts from device and then return facts
        :param int timeout: timeout in seconds to wait for facts from given device id
        :return: DeviceSchema
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.retrieve_iceberg_device_device_facts_by_id_with_http_info(device_id, **kwargs)  # noqa: E501
        else:
            (data) = self.retrieve_iceberg_device_device_facts_by_id_with_http_info(device_id, **kwargs)  # noqa: E501
            return data

    def retrieve_iceberg_device_device_facts_by_id_with_http_info(self, device_id, **kwargs):  # noqa: E501
        """Get a device's facts.  # noqa: E501

        Get the fact details of a device by its `device-id`.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.retrieve_iceberg_device_device_facts_by_id_with_http_info(device_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str device_id: ID of device-id (required)
        :param str x_iam_token: authentication header object
        :param bool working: true queries un-committed configuration
        :param bool update: true will first update facts from device and then return facts
        :param int timeout: timeout in seconds to wait for facts from given device id
        :return: DeviceSchema
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['device_id', 'x_iam_token', 'working', 'update', 'timeout']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method retrieve_iceberg_device_device_facts_by_id" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'device_id' is set
        if ('device_id' not in params or
                params['device_id'] is None):
            raise ValueError("Missing the required parameter `device_id` when calling `retrieve_iceberg_device_device_facts_by_id`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'device_id' in params:
            path_params['device_id'] = params['device_id']  # noqa: E501

        query_params = []
        if 'working' in params:
            query_params.append(('working', params['working']))  # noqa: E501
        if 'update' in params:
            query_params.append(('update', params['update']))  # noqa: E501
        if 'timeout' in params:
            query_params.append(('timeout', params['timeout']))  # noqa: E501

        header_params = {}
        if 'x_iam_token' in params:
            header_params['x-iam-token'] = params['x_iam_token']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json', 'multipart/form-data'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/config/device/{device_id}/facts/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='DeviceSchema',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def retrieve_iceberg_devices_devices_facts(self, **kwargs):  # noqa: E501
        """Get devices facts.  # noqa: E501

        Get the fact details of every device  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.retrieve_iceberg_devices_devices_facts(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str x_iam_token: authentication header object
        :param bool working: true queries un-committed configuration
        :param bool update: true will first update facts from device and then return facts
        :param int timeout: timeout in seconds to wait for facts from every device
        :return: DeviceSchema
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.retrieve_iceberg_devices_devices_facts_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.retrieve_iceberg_devices_devices_facts_with_http_info(**kwargs)  # noqa: E501
            return data

    def retrieve_iceberg_devices_devices_facts_with_http_info(self, **kwargs):  # noqa: E501
        """Get devices facts.  # noqa: E501

        Get the fact details of every device  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.retrieve_iceberg_devices_devices_facts_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str x_iam_token: authentication header object
        :param bool working: true queries un-committed configuration
        :param bool update: true will first update facts from device and then return facts
        :param int timeout: timeout in seconds to wait for facts from every device
        :return: DeviceSchema
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['x_iam_token', 'working', 'update', 'timeout']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method retrieve_iceberg_devices_devices_facts" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'working' in params:
            query_params.append(('working', params['working']))  # noqa: E501
        if 'update' in params:
            query_params.append(('update', params['update']))  # noqa: E501
        if 'timeout' in params:
            query_params.append(('timeout', params['timeout']))  # noqa: E501

        header_params = {}
        if 'x_iam_token' in params:
            header_params['x-iam-token'] = params['x_iam_token']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json', 'multipart/form-data'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/config/devices/facts/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='DeviceSchema',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def retrieve_iceberg_devices_facts_by_group(self, device_group_name, **kwargs):  # noqa: E501
        """Get a devices facts for given group.  # noqa: E501

        Get the fact details of every device under given group  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.retrieve_iceberg_devices_facts_by_group(device_group_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str device_group_name: ID of group (required)
        :param str x_iam_token: authentication header object
        :param bool working: true queries un-committed configuration
        :param bool update: true will first update facts from device and then return facts
        :param int timeout: timeout in seconds to wait for facts from every device
        :return: DeviceSchema
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.retrieve_iceberg_devices_facts_by_group_with_http_info(device_group_name, **kwargs)  # noqa: E501
        else:
            (data) = self.retrieve_iceberg_devices_facts_by_group_with_http_info(device_group_name, **kwargs)  # noqa: E501
            return data

    def retrieve_iceberg_devices_facts_by_group_with_http_info(self, device_group_name, **kwargs):  # noqa: E501
        """Get a devices facts for given group.  # noqa: E501

        Get the fact details of every device under given group  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.retrieve_iceberg_devices_facts_by_group_with_http_info(device_group_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str device_group_name: ID of group (required)
        :param str x_iam_token: authentication header object
        :param bool working: true queries un-committed configuration
        :param bool update: true will first update facts from device and then return facts
        :param int timeout: timeout in seconds to wait for facts from every device
        :return: DeviceSchema
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['device_group_name', 'x_iam_token', 'working', 'update', 'timeout']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method retrieve_iceberg_devices_facts_by_group" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'device_group_name' is set
        if ('device_group_name' not in params or
                params['device_group_name'] is None):
            raise ValueError("Missing the required parameter `device_group_name` when calling `retrieve_iceberg_devices_facts_by_group`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'device_group_name' in params:
            path_params['device_group_name'] = params['device_group_name']  # noqa: E501

        query_params = []
        if 'working' in params:
            query_params.append(('working', params['working']))  # noqa: E501
        if 'update' in params:
            query_params.append(('update', params['update']))  # noqa: E501
        if 'timeout' in params:
            query_params.append(('timeout', params['timeout']))  # noqa: E501

        header_params = {}
        if 'x_iam_token' in params:
            header_params['x-iam-token'] = params['x_iam_token']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json', 'multipart/form-data'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/config/device-group/{device_group_name}/facts/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='DeviceSchema',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class NetworkgroupSchemaLogging(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'log_level': 'str',
        'non_sensor_rules': 'DevicegroupSchemaLoggingNonsensorrules',
        'reports_generation': 'DevicegroupSchemaLoggingReportsgeneration',
        'trigger_evaluation': 'DevicegroupSchemaLoggingTriggerevaluation',
        'ml_model_builder': 'DevicegroupSchemaLoggingMLmodelbuilder',
        'resource_discovery': 'DevicegroupSchemaLoggingResourcediscovery'
    }

    attribute_map = {
        'log_level': 'log-level',
        'non_sensor_rules': 'non-sensor-rules',
        'reports_generation': 'reports-generation',
        'trigger_evaluation': 'trigger-evaluation',
        'ml_model_builder': 'ML-model-builder',
        'resource_discovery': 'resource-discovery'
    }

    def __init__(self, log_level=None, non_sensor_rules=None, reports_generation=None, trigger_evaluation=None, ml_model_builder=None, resource_discovery=None):  # noqa: E501
        """NetworkgroupSchemaLogging - a model defined in Swagger"""  # noqa: E501

        self._log_level = None
        self._non_sensor_rules = None
        self._reports_generation = None
        self._trigger_evaluation = None
        self._ml_model_builder = None
        self._resource_discovery = None
        self.discriminator = None

        if log_level is not None:
            self.log_level = log_level
        if non_sensor_rules is not None:
            self.non_sensor_rules = non_sensor_rules
        if reports_generation is not None:
            self.reports_generation = reports_generation
        if trigger_evaluation is not None:
            self.trigger_evaluation = trigger_evaluation
        if ml_model_builder is not None:
            self.ml_model_builder = ml_model_builder
        if resource_discovery is not None:
            self.resource_discovery = resource_discovery

    @property
    def log_level(self):
        """Gets the log_level of this NetworkgroupSchemaLogging.  # noqa: E501

        Global log level  # noqa: E501

        :return: The log_level of this NetworkgroupSchemaLogging.  # noqa: E501
        :rtype: str
        """
        return self._log_level

    @log_level.setter
    def log_level(self, log_level):
        """Sets the log_level of this NetworkgroupSchemaLogging.

        Global log level  # noqa: E501

        :param log_level: The log_level of this NetworkgroupSchemaLogging.  # noqa: E501
        :type: str
        """
        allowed_values = ["error", "debug", "warn", "info"]  # noqa: E501
        if log_level not in allowed_values:
            raise ValueError(
                "Invalid value for `log_level` ({0}), must be one of {1}"  # noqa: E501
                .format(log_level, allowed_values)
            )

        self._log_level = log_level

    @property
    def non_sensor_rules(self):
        """Gets the non_sensor_rules of this NetworkgroupSchemaLogging.  # noqa: E501


        :return: The non_sensor_rules of this NetworkgroupSchemaLogging.  # noqa: E501
        :rtype: DevicegroupSchemaLoggingNonsensorrules
        """
        return self._non_sensor_rules

    @non_sensor_rules.setter
    def non_sensor_rules(self, non_sensor_rules):
        """Sets the non_sensor_rules of this NetworkgroupSchemaLogging.


        :param non_sensor_rules: The non_sensor_rules of this NetworkgroupSchemaLogging.  # noqa: E501
        :type: DevicegroupSchemaLoggingNonsensorrules
        """

        self._non_sensor_rules = non_sensor_rules

    @property
    def reports_generation(self):
        """Gets the reports_generation of this NetworkgroupSchemaLogging.  # noqa: E501


        :return: The reports_generation of this NetworkgroupSchemaLogging.  # noqa: E501
        :rtype: DevicegroupSchemaLoggingReportsgeneration
        """
        return self._reports_generation

    @reports_generation.setter
    def reports_generation(self, reports_generation):
        """Sets the reports_generation of this NetworkgroupSchemaLogging.


        :param reports_generation: The reports_generation of this NetworkgroupSchemaLogging.  # noqa: E501
        :type: DevicegroupSchemaLoggingReportsgeneration
        """

        self._reports_generation = reports_generation

    @property
    def trigger_evaluation(self):
        """Gets the trigger_evaluation of this NetworkgroupSchemaLogging.  # noqa: E501


        :return: The trigger_evaluation of this NetworkgroupSchemaLogging.  # noqa: E501
        :rtype: DevicegroupSchemaLoggingTriggerevaluation
        """
        return self._trigger_evaluation

    @trigger_evaluation.setter
    def trigger_evaluation(self, trigger_evaluation):
        """Sets the trigger_evaluation of this NetworkgroupSchemaLogging.


        :param trigger_evaluation: The trigger_evaluation of this NetworkgroupSchemaLogging.  # noqa: E501
        :type: DevicegroupSchemaLoggingTriggerevaluation
        """

        self._trigger_evaluation = trigger_evaluation

    @property
    def ml_model_builder(self):
        """Gets the ml_model_builder of this NetworkgroupSchemaLogging.  # noqa: E501


        :return: The ml_model_builder of this NetworkgroupSchemaLogging.  # noqa: E501
        :rtype: DevicegroupSchemaLoggingMLmodelbuilder
        """
        return self._ml_model_builder

    @ml_model_builder.setter
    def ml_model_builder(self, ml_model_builder):
        """Sets the ml_model_builder of this NetworkgroupSchemaLogging.


        :param ml_model_builder: The ml_model_builder of this NetworkgroupSchemaLogging.  # noqa: E501
        :type: DevicegroupSchemaLoggingMLmodelbuilder
        """

        self._ml_model_builder = ml_model_builder

    @property
    def resource_discovery(self):
        """Gets the resource_discovery of this NetworkgroupSchemaLogging.  # noqa: E501


        :return: The resource_discovery of this NetworkgroupSchemaLogging.  # noqa: E501
        :rtype: DevicegroupSchemaLoggingResourcediscovery
        """
        return self._resource_discovery

    @resource_discovery.setter
    def resource_discovery(self, resource_discovery):
        """Sets the resource_discovery of this NetworkgroupSchemaLogging.


        :param resource_discovery: The resource_discovery of this NetworkgroupSchemaLogging.  # noqa: E501
        :type: DevicegroupSchemaLoggingResourcediscovery
        """

        self._resource_discovery = resource_discovery

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(NetworkgroupSchemaLogging, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NetworkgroupSchemaLogging):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import time
import copy

from jnpr.healthbot.swagger.models.playbook_schema import PlaybookSchema
from jnpr.healthbot.swagger.models.rule_schema_variable import RuleSchemaVariable
from jnpr.healthbot.modules import BaseModule
from jnpr.healthbot.modules.rules import Rule
from jnpr.healthbot.modules.devices import Device, DeviceGroup

from jnpr.healthbot.exception import SchemaError, NotFoundError

import logging
logger = logging.getLogger(__file__)


class Playbook(BaseModule):

    def __init__(self, hbot):
        """
        :param object hbot: :class:`jnpr.healthbot.HealthBotClient` client instance
        """

        super().__init__(hbot)

    def add(self, schema: PlaybookSchema = None, **kwargs):
        """
        Add playbook

        :param object schema: `PlaybookSchema <jnpr.healthbot.swagger.models.html#playbookschema>`_
        :param object kwargs: key values, which can be used to create
            PlaybookSchema
            Check `PlaybookSchema <jnpr.healthbot.swagger.models.html#playbookschema>`_
            for details about which all keys can be used

        Example:
        ::
            from jnpr.healthbot import HealthBotClient

            with HealthBotClient('xx.xxx.x.xx', 'xxxx', 'xxxx') as hb:
                hb.playbook.add(playbook_name="HbEZ-example",
                        rules = ['protocol.infra/check-task-momory-usage'])

            # or
            from jnpr.healthbot import HealthBotClient
            from jnpr.healthbot import PlaybookSchema

            with HealthBotClient('xx.xxx.x.xx', 'xxxx', 'xxxx') as hb:
                pbs = PlaybookSchema(playbook_name="HbEZ-example",
                        rules = ['protocol.infra/check-task-momory-usage'])
                hb.playbook.add(pbs)

        Returns:
            True if action successful
        """
        if schema is None:
            schema = PlaybookSchema(**kwargs)

        payload = self.hbot._create_payload(schema)

        payload_url = self.hbot.urlfor.playbook(payload['playbook-name'])
        resp = self.api.get(payload_url + self.hbot.apiopt_candidate)
        if resp.ok:
            return True
        response = self.api.post(payload_url, json=payload)
        if response.status_code != 200:
            logger.error(response.text)
        response.raise_for_status()

        return True

    def delete(self, playbook_name: str):
        """
        Delete playbook

        :param str playbook_name: The playbook name to deleted

        Example:
        ::
            from jnpr.healthbot import HealthBotClient

            with HealthBotClient('xx.xxx.x.xx', 'xxxx', 'xxxx') as hb:
                hb.playbook.delete('linecard-kpis-playbook')

        """

        payload = {'playbook-name': playbook_name}
        rule_url = self.hbot.urlfor.playbook(playbook_name)
        response = self.api.delete(rule_url, json=payload)
        if response.status_code != 204:
            logger.error(response.text)
        response.raise_for_status()
        return True

    def get(self, playbook_name: str = None, uncommitted: bool = True):
        """
        get playbook details

        :param str playbook_name: Name of the playbook
        :param bool uncommitted: True includes fetches uncommitted changes,
                False restricts data set to only committed changes

        Example:
        ::
            from jnpr.healthbot import HealthBotClient

            with HealthBotClient('xx.xxx.x.xx', 'xxxx', 'xxxx') as hb:
                print(hb.playbook.get('linecard-kpis-playbook'))

                # for all
                print(hb.playbook.get())

        """
        if playbook_name is not None:
            playbook_url = self.hbot.urlfor.playbook(playbook_name)
            if uncommitted:
                playbook_url += self.hbot.apiopt_candidate
            response = self.api.get(playbook_url)
            if response.status_code != 200:
                logger.error(response.text)
            response.raise_for_status()
            return self.hbot._create_schema(response, PlaybookSchema)
        else:
            playbook_list = []

            playbooks_list_url = self.hbot.urlfor.playbooks()
            if uncommitted:
                playbooks_list_url += self.hbot.apiopt_candidate
            resp = self.api.get(playbooks_list_url)

            if resp.status_code == 404:
                return False

            existing_playbooks = resp.json()['playbook']
            for playbook in existing_playbooks:
                obj = self.hbot._create_schema(playbook, PlaybookSchema)
                playbook_list.append(obj)

            return playbook_list

    def update(self, schema: PlaybookSchema = None, **kwargs):
        """
        Update `PlaybookSchema <jnpr.healthbot.swagger.models.html#playbookschema>`_ for
        given playbook schema

        Passing Schema invoke `put` and kwargs `post`

        :param obj schema: `PlaybookSchema <jnpr.healthbot.swagger.models.html#playbookschema>`_
        :param object kwargs: key values, which can be used to create
            PlaybookSchema
            Check `PlaybookSchema <jnpr.healthbot.swagger.models.html#playbookschema>`_
            for details about which all keys can be used

        Example:
        ::

            from jnpr.healthbot import HealthBotClient

            with HealthBotClient('xx.xxx.x.xx', 'xxxx', 'xxxx') as hb:
                schemaObj = hb.playbook.get('xyz')
                schemaObj.description = 'changed description'
                hb.playbook.update(schemaObj)

        :returns: True when OK
        """
        if schema is None:
            schema = PlaybookSchema(**kwargs)
            call = self.api.post
        else:
            if not isinstance(schema, PlaybookSchema):
                raise SchemaError(PlaybookSchema)
            call = self.api.put
        payload = self.hbot._create_payload(schema)
        name = schema.playbook_name
        playbook_url = self.hbot.urlfor.playbook(name)
        response = call(playbook_url, json=payload)
        if response.status_code != 200:
            logger.error(response.text)
        response.raise_for_status()
        return True

    def upload_playbook_file(self, filename):
        """

        :param filename: File to be loaded
        :return: return True of OK
        """
        return self.hbot.upload_helper_file(filename)


class _RuleVariableClass(object):

    def __init__(self, values):
        super(_RuleVariableClass, self).__setattr__('_values', values)
        super(_RuleVariableClass, self).__setattr__('_variable_value', [])
        for item in values:
            name = item['name'].replace('-', '_')
            self.__dict__[name] = RuleSchemaVariable(**item).value
            self.__dict__['_{}_schema'.format(
                name)] = RuleSchemaVariable(**item)

    def __setattr__(self, key, value):
        if key in self.__dict__:
            # To make sure value is as per rule/type defined
            self.__dict__['_{}_schema'.format(key)].value = value
            # value should be of string type in DeviceGroupVariableSchema
            self._variable_value.append({'name': key.replace('_', '-'),
                                         'value': str(value)})

    def __repr__(self):
        return str(self._values)


class PlayBookInstanceBuilder(Playbook):
    def __init__(self, hbot, playbook: str, instance: str = None,
                 device_group_name: str = None):
        """
        Help in building and applying playbook instance

        :param hbot: HealthBOtClient instance
        :param playbook: Playbook name for which instance need to be created
        :param instance: Playbook instance name
        :param device_group_name: Device group which will be associated with instance

        Example:
        ::
            from jnpr.healthbot import HealthBotClient

            with HealthBotClient('xx.xxx.x.xx', 'xxxx', 'xxxx') as hb:

                from jnpr.healthbot import PlayBookInstanceBuilder
                pbb = PlayBookInstanceBuilder(hb, 'forwarding-table-summary', 'HbEZ-instance', 'Core')

                variable = pbb.rule_variables["protocol.routesummary/check-fib-summary"]
                variable.route_address_family = 'pqr'
                variable.route_count_threshold = 100

                # Apply variable to given device(s)
                pbb.apply(device_ids=['vmx'])

                #clear all the variable if you want to set it something else for group or other device(s)
                pbb.clear()

                variable = pbb.rule_variables["protocol.routesummary/check-fib-summary"]
                variable.route_address_family = 'abc'
                variable.route_count_threshold = 200

                pbb.apply()

                hb.commit()
        """
        Playbook.__init__(self, hbot)
        self.hbot = hbot
        self.playbook = playbook
        self.instance_id = instance
        self.device_group_name = device_group_name
        self._rule_variables = dict()
        self._rule_variables = self.rule_variables

    def apply(self, device_ids: list = None, commit: bool = False):
        """
        Apply the playbook instance

        :param device_ids: if the rule variables need to be associated for
            given device id(s). Default to device group
        :param commit: Pass true if need to commit the changes

        Example:
        ::

            from jnpr.healthbot import PlayBookInstanceBuilder
            pbb = PlayBookInstanceBuilder(hb, 'forwarding-table-summary', 'HbEZ-instance', 'Core')
            pbb.apply()

        :return: True if all OK
        """
        if self.device_group_name is None:
            raise RuntimeError("PlayBookInstanceBuilder object device group attribute not provided")
        device_instance = Device(self.hbot)
        device_group_instance = DeviceGroup(self.hbot)
        device_group = device_group_instance.get(self.device_group_name)
        if device_group is None:
            raise NotFoundError({'detail': 'No details for device group: {}'.format(
                self.device_group_name), 'status': 404})
        if device_ids is not None:
            for device_id in device_ids:
                if device_id not in device_group.devices:
                    raise RuntimeError("Given device id '{}' is not present".
                                       format(device_id))
                device = device_instance.get(device_id)
                device.variable = self.device_variable
                if not device_instance.update(device):
                    logger.error(
                        "Not able to update '{}' device id".format(device_id))
            device_group_variable = []

            for item in self.device_variable:
                tmp = copy.copy(item)
                tmp.pop('variable-value', None)
                device_group_variable.append(tmp)
            existing_variable = device_group.variable
            if existing_variable is None:
                device_group.variable = device_group_variable
            else:
                existing_variable.extend(device_group_variable)
                device_group.variable = existing_variable
            if not device_group_instance.update(device_group):
                logger.error(
                    "Not able to update '{}' device group".format(device_group))
                return False
        else:
            existing_variable = device_group.variable
            if existing_variable is None:
                device_group.variable = self.device_variable
            else:
                existing_variable.extend(self.device_variable)
                device_group.variable = existing_variable
            existing_playbooks = device_group.playbooks or []
            existing_playbooks.append(self.playbook)
            device_group.playbooks = existing_playbooks
            if not device_group_instance.update(device_group):
                logger.error(
                    "Not able to update '{}' device group".format(device_group))
                return False
        if commit:
            self.hbot.commit()
            return True
        return True

    def clear(self):
        """
        Clear the old set values to rule variables
        :return: None
        """
        self._rule_variables = dict()
        self._rule_variables = self.rule_variables

    def delete(self):
        """
        Delete playbook instance

        Example:
        ::

            from jnpr.healthbot import PlayBookInstanceBuilder
            pbb = PlayBookInstanceBuilder(hb, 'forwarding-table-summary', 'HbEZ-instance', 'Core')
            pbb.delete()

        :return: True if success
        """
        delete_playbook_from_group = True
        if self.device_group_name is None:
            raise RuntimeError("PlayBookInstanceBuilder object device group attribute not provided")
        device_instance = Device(self.hbot)
        device_group_instance = DeviceGroup(self.hbot)
        device_group = device_group_instance.get(self.device_group_name)
        if device_group is None:
            raise NotFoundError({'detail': 'No details for device group: {}'.format(
                self.device_group_name), 'status': 404})
        for device_id in device_group.devices:
            device = device_instance.get(device_id)
            existing_variables = device.variable or []
            if len(existing_variables) > 0:
                changed = False
                for variable in existing_variables:
                    if self.playbook == variable.get('playbook'):
                        if self.instance_id == variable.get('instance-id'):
                            device.variable.remove(variable)
                            changed = True
                        else:
                            delete_playbook_from_group = False
                if changed and not device_instance.update(device):
                    logger.error(
                        "Not able to update '{}' device id".format(device_id))
                    return False

        existing_variables = copy.copy(device_group.variable) or []
        changed = False
        if len(existing_variables) > 0:
            for variable in existing_variables:
                if self.playbook == variable.get('playbook'):
                    if self.instance_id == variable.get('instance-id'):
                        device_group.variable.remove(variable)
                        changed = True
                    else:
                        delete_playbook_from_group = False
            if changed and not device_group_instance.update(device_group):
                logger.error(
                    "Not able to update '{}' device id".format(device_id))
                return False
        if not changed:
            logger.error(
                "Does not exist '{}' instance-id".format(self.instance_id))
            return False
        if delete_playbook_from_group:
            update_playbooks = device_group.playbooks or []
            if self.playbook in update_playbooks:
                update_playbooks.remove(self.playbook)
                device_group.playbook = update_playbooks
                if not device_group_instance.update(device_group):
                    logger.error(
                        "Not able to update '{}' device group".format(device_group))
                    return False
        return True

    @property
    def playbook_schema(self):
        return self._get_playbook_schema()

    @playbook_schema.setter
    def playbook_schema(self, value):
        """ read-only property """
        raise RuntimeError("playbook_schema is read-only!")

    @property
    def rules(self):
        return self.playbook_schema.to_dict().get('rules', [])

    @rules.setter
    def rules(self, value):
        """ read-only property """
        raise RuntimeError("rules is read-only!")

    @property
    def rule_variables(self):
        for i in self.rules:
            if i not in self._rule_variables:
                variable = Rule(self.hbot).get(*i.split('/')).variable
                if variable is not None:
                    self._rule_variables[i] = _RuleVariableClass(variable)
                else:
                    self._rule_variables[i] = None
        return self._rule_variables

    @rule_variables.setter
    def rule_variables(self, value):
        """ read-only property """
        raise RuntimeError("rules_variables is read-only!")

    @property
    def device_variable(self):
        variable = []
        for key, val in self._rule_variables.items():
            tmp = dict()
            tmp['@'] = {'changed-seconds': int(time.time())}
            tmp['instance-id'] = self.instance_id
            tmp['rule'] = key
            tmp['playbook'] = self.playbook
            variable_value = val._variable_value if val is not None else []
            tmp['variable-value'] = variable_value
            variable.append(tmp)
        return variable

    def _get_playbook_schema(self):
        """
        Get PlaybookSchema for given playbook name

        :return: PlaybookSchema `PlaybookSchema <jnpr.healthbot.swagger.models.html#playbookschema>`_
        """
        # get playbook schema first
        try:
            logger.debug("Getting details of playbook '{}'".format(
                self.playbook))
            return self.get(self.playbook)
        except Exception as ex:
            logger.error("Got exception '{}' while getting details of "
                         "playbook '{}'".format(ex, self.playbook_name))
            raise ex



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class PatternSchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'constant': 'list[PatternSchemaConstant]',
        'description': 'str',
        'event_id': 'str',
        'field': 'list[PatternSchemaField]',
        'filter': 'str',
        'filter_type': 'str',
        'key_fields': 'list[str]',
        'name': 'str'
    }

    attribute_map = {
        'constant': 'constant',
        'description': 'description',
        'event_id': 'event-id',
        'field': 'field',
        'filter': 'filter',
        'filter_type': 'filter-type',
        'key_fields': 'key-fields',
        'name': 'name'
    }

    def __init__(self, constant=None, description=None, event_id=None, field=None, filter=None, filter_type=None, key_fields=None, name=None):  # noqa: E501
        """PatternSchema - a model defined in Swagger"""  # noqa: E501

        self._constant = None
        self._description = None
        self._event_id = None
        self._field = None
        self._filter = None
        self._filter_type = None
        self._key_fields = None
        self._name = None
        self.discriminator = None

        if constant is not None:
            self.constant = constant
        if description is not None:
            self.description = description
        self.event_id = event_id
        if field is not None:
            self.field = field
        if filter is not None:
            self.filter = filter
        if filter_type is not None:
            self.filter_type = filter_type
        if key_fields is not None:
            self.key_fields = key_fields
        self.name = name

    @property
    def constant(self):
        """Gets the constant of this PatternSchema.  # noqa: E501

        Constant details  # noqa: E501

        :return: The constant of this PatternSchema.  # noqa: E501
        :rtype: list[PatternSchemaConstant]
        """
        return self._constant

    @constant.setter
    def constant(self, constant):
        """Sets the constant of this PatternSchema.

        Constant details  # noqa: E501

        :param constant: The constant of this PatternSchema.  # noqa: E501
        :type: list[PatternSchemaConstant]
        """

        self._constant = constant

    @property
    def description(self):
        """Gets the description of this PatternSchema.  # noqa: E501

        Pattern description  # noqa: E501

        :return: The description of this PatternSchema.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this PatternSchema.

        Pattern description  # noqa: E501

        :param description: The description of this PatternSchema.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def event_id(self):
        """Gets the event_id of this PatternSchema.  # noqa: E501

        Event id that identifies a log uniquely. Field names also can be part of event-id. Example my-event+$field1  # noqa: E501

        :return: The event_id of this PatternSchema.  # noqa: E501
        :rtype: str
        """
        return self._event_id

    @event_id.setter
    def event_id(self, event_id):
        """Sets the event_id of this PatternSchema.

        Event id that identifies a log uniquely. Field names also can be part of event-id. Example my-event+$field1  # noqa: E501

        :param event_id: The event_id of this PatternSchema.  # noqa: E501
        :type: str
        """
        if event_id is None:
            raise ValueError("Invalid value for `event_id`, must not be `None`")  # noqa: E501

        self._event_id = event_id

    @property
    def field(self):
        """Gets the field of this PatternSchema.  # noqa: E501

        Field details  # noqa: E501

        :return: The field of this PatternSchema.  # noqa: E501
        :rtype: list[PatternSchemaField]
        """
        return self._field

    @field.setter
    def field(self, field):
        """Sets the field of this PatternSchema.

        Field details  # noqa: E501

        :param field: The field of this PatternSchema.  # noqa: E501
        :type: list[PatternSchemaField]
        """

        self._field = field

    @property
    def filter(self):
        """Gets the filter of this PatternSchema.  # noqa: E501

        Filter to match a log line  # noqa: E501

        :return: The filter of this PatternSchema.  # noqa: E501
        :rtype: str
        """
        return self._filter

    @filter.setter
    def filter(self, filter):
        """Sets the filter of this PatternSchema.

        Filter to match a log line  # noqa: E501

        :param filter: The filter of this PatternSchema.  # noqa: E501
        :type: str
        """

        self._filter = filter

    @property
    def filter_type(self):
        """Gets the filter_type of this PatternSchema.  # noqa: E501

        Filter type, default is grok  # noqa: E501

        :return: The filter_type of this PatternSchema.  # noqa: E501
        :rtype: str
        """
        return self._filter_type

    @filter_type.setter
    def filter_type(self, filter_type):
        """Sets the filter_type of this PatternSchema.

        Filter type, default is grok  # noqa: E501

        :param filter_type: The filter_type of this PatternSchema.  # noqa: E501
        :type: str
        """
        allowed_values = ["grok"]  # noqa: E501
        if filter_type not in allowed_values:
            raise ValueError(
                "Invalid value for `filter_type` ({0}), must be one of {1}"  # noqa: E501
                .format(filter_type, allowed_values)
            )

        self._filter_type = filter_type

    @property
    def key_fields(self):
        """Gets the key_fields of this PatternSchema.  # noqa: E501


        :return: The key_fields of this PatternSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._key_fields

    @key_fields.setter
    def key_fields(self, key_fields):
        """Sets the key_fields of this PatternSchema.


        :param key_fields: The key_fields of this PatternSchema.  # noqa: E501
        :type: list[str]
        """

        self._key_fields = key_fields

    @property
    def name(self):
        """Gets the name of this PatternSchema.  # noqa: E501

        Name of a pattern. Should be of pattern [a-zA-Z][a-zA-Z0-9_-]*  # noqa: E501

        :return: The name of this PatternSchema.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this PatternSchema.

        Name of a pattern. Should be of pattern [a-zA-Z][a-zA-Z0-9_-]*  # noqa: E501

        :param name: The name of this PatternSchema.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501
        if name is not None and len(name) > 64:
            raise ValueError("Invalid value for `name`, length must be less than or equal to `64`")  # noqa: E501
        if name is not None and not re.search(r'^[a-zA-Z][a-zA-Z0-9_-]*$', name):  # noqa: E501
            raise ValueError(r"Invalid value for `name`, must be a follow pattern or equal to `/^[a-zA-Z][a-zA-Z0-9_-]*$/`")  # noqa: E501

        self._name = name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PatternSchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PatternSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class TopicSchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'description': 'str',
        'resource': 'list[TopicSchemaResource]',
        'rule': 'list[RuleSchema]',
        'sub_topics': 'list[str]',
        'synopsis': 'str',
        'topic_name': 'str'
    }

    attribute_map = {
        'description': 'description',
        'resource': 'resource',
        'rule': 'rule',
        'sub_topics': 'sub-topics',
        'synopsis': 'synopsis',
        'topic_name': 'topic-name'
    }

    def __init__(self, description=None, resource=None, rule=None, sub_topics=None, synopsis=None, topic_name=None):  # noqa: E501
        """TopicSchema - a model defined in Swagger"""  # noqa: E501

        self._description = None
        self._resource = None
        self._rule = None
        self._sub_topics = None
        self._synopsis = None
        self._topic_name = None
        self.discriminator = None

        if description is not None:
            self.description = description
        if resource is not None:
            self.resource = resource
        if rule is not None:
            self.rule = rule
        if sub_topics is not None:
            self.sub_topics = sub_topics
        if synopsis is not None:
            self.synopsis = synopsis
        self.topic_name = topic_name

    @property
    def description(self):
        """Gets the description of this TopicSchema.  # noqa: E501

        Description about this topic  # noqa: E501

        :return: The description of this TopicSchema.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this TopicSchema.

        Description about this topic  # noqa: E501

        :param description: The description of this TopicSchema.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def resource(self):
        """Gets the resource of this TopicSchema.  # noqa: E501


        :return: The resource of this TopicSchema.  # noqa: E501
        :rtype: list[TopicSchemaResource]
        """
        return self._resource

    @resource.setter
    def resource(self, resource):
        """Sets the resource of this TopicSchema.


        :param resource: The resource of this TopicSchema.  # noqa: E501
        :type: list[TopicSchemaResource]
        """

        self._resource = resource

    @property
    def rule(self):
        """Gets the rule of this TopicSchema.  # noqa: E501


        :return: The rule of this TopicSchema.  # noqa: E501
        :rtype: list[RuleSchema]
        """
        return self._rule

    @rule.setter
    def rule(self, rule):
        """Sets the rule of this TopicSchema.


        :param rule: The rule of this TopicSchema.  # noqa: E501
        :type: list[RuleSchema]
        """

        self._rule = rule

    @property
    def sub_topics(self):
        """Gets the sub_topics of this TopicSchema.  # noqa: E501


        :return: The sub_topics of this TopicSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._sub_topics

    @sub_topics.setter
    def sub_topics(self, sub_topics):
        """Sets the sub_topics of this TopicSchema.


        :param sub_topics: The sub_topics of this TopicSchema.  # noqa: E501
        :type: list[str]
        """

        self._sub_topics = sub_topics

    @property
    def synopsis(self):
        """Gets the synopsis of this TopicSchema.  # noqa: E501

        Short description about this topic  # noqa: E501

        :return: The synopsis of this TopicSchema.  # noqa: E501
        :rtype: str
        """
        return self._synopsis

    @synopsis.setter
    def synopsis(self, synopsis):
        """Sets the synopsis of this TopicSchema.

        Short description about this topic  # noqa: E501

        :param synopsis: The synopsis of this TopicSchema.  # noqa: E501
        :type: str
        """

        self._synopsis = synopsis

    @property
    def topic_name(self):
        """Gets the topic_name of this TopicSchema.  # noqa: E501

        Name of the topic. Should be of pattern [a-z][a-z-]*(\\.{1}[a-z0-9-]+)*  # noqa: E501

        :return: The topic_name of this TopicSchema.  # noqa: E501
        :rtype: str
        """
        return self._topic_name

    @topic_name.setter
    def topic_name(self, topic_name):
        """Sets the topic_name of this TopicSchema.

        Name of the topic. Should be of pattern [a-z][a-z-]*(\\.{1}[a-z0-9-]+)*  # noqa: E501

        :param topic_name: The topic_name of this TopicSchema.  # noqa: E501
        :type: str
        """
        if topic_name is None:
            raise ValueError("Invalid value for `topic_name`, must not be `None`")  # noqa: E501
        if topic_name is not None and len(topic_name) > 64:
            raise ValueError("Invalid value for `topic_name`, length must be less than or equal to `64`")  # noqa: E501
        if topic_name is not None and not re.search(r'^[a-z][a-z-]*(\\.{1}[a-z0-9-]+)*$', topic_name):  # noqa: E501
            raise ValueError(r"Invalid value for `topic_name`, must be a follow pattern or equal to `/^[a-z][a-z-]*(\\.{1}[a-z0-9-]+)*$/`")  # noqa: E501

        self._topic_name = topic_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(TopicSchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TopicSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class SchedulerSchemaRunfor(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'days': 'int',
        'hours': 'int',
        'minutes': 'int'
    }

    attribute_map = {
        'days': 'days',
        'hours': 'hours',
        'minutes': 'minutes'
    }

    def __init__(self, days=None, hours=None, minutes=None):  # noqa: E501
        """SchedulerSchemaRunfor - a model defined in Swagger"""  # noqa: E501

        self._days = None
        self._hours = None
        self._minutes = None
        self.discriminator = None

        if days is not None:
            self.days = days
        if hours is not None:
            self.hours = hours
        if minutes is not None:
            self.minutes = minutes

    @property
    def days(self):
        """Gets the days of this SchedulerSchemaRunfor.  # noqa: E501

        Duration of time in days  # noqa: E501

        :return: The days of this SchedulerSchemaRunfor.  # noqa: E501
        :rtype: int
        """
        return self._days

    @days.setter
    def days(self, days):
        """Sets the days of this SchedulerSchemaRunfor.

        Duration of time in days  # noqa: E501

        :param days: The days of this SchedulerSchemaRunfor.  # noqa: E501
        :type: int
        """
        if days is not None and days > 65535:  # noqa: E501
            raise ValueError("Invalid value for `days`, must be a value less than or equal to `65535`")  # noqa: E501
        if days is not None and days < 1:  # noqa: E501
            raise ValueError("Invalid value for `days`, must be a value greater than or equal to `1`")  # noqa: E501

        self._days = days

    @property
    def hours(self):
        """Gets the hours of this SchedulerSchemaRunfor.  # noqa: E501

        Duration of time in hours  # noqa: E501

        :return: The hours of this SchedulerSchemaRunfor.  # noqa: E501
        :rtype: int
        """
        return self._hours

    @hours.setter
    def hours(self, hours):
        """Sets the hours of this SchedulerSchemaRunfor.

        Duration of time in hours  # noqa: E501

        :param hours: The hours of this SchedulerSchemaRunfor.  # noqa: E501
        :type: int
        """
        if hours is not None and hours > 65535:  # noqa: E501
            raise ValueError("Invalid value for `hours`, must be a value less than or equal to `65535`")  # noqa: E501
        if hours is not None and hours < 1:  # noqa: E501
            raise ValueError("Invalid value for `hours`, must be a value greater than or equal to `1`")  # noqa: E501

        self._hours = hours

    @property
    def minutes(self):
        """Gets the minutes of this SchedulerSchemaRunfor.  # noqa: E501

        Duration of time in minutes  # noqa: E501

        :return: The minutes of this SchedulerSchemaRunfor.  # noqa: E501
        :rtype: int
        """
        return self._minutes

    @minutes.setter
    def minutes(self, minutes):
        """Sets the minutes of this SchedulerSchemaRunfor.

        Duration of time in minutes  # noqa: E501

        :param minutes: The minutes of this SchedulerSchemaRunfor.  # noqa: E501
        :type: int
        """
        if minutes is not None and minutes > 65535:  # noqa: E501
            raise ValueError("Invalid value for `minutes`, must be a value less than or equal to `65535`")  # noqa: E501
        if minutes is not None and minutes < 1:  # noqa: E501
            raise ValueError("Invalid value for `minutes`, must be a value greater than or equal to `1`")  # noqa: E501

        self._minutes = minutes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(SchedulerSchemaRunfor, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SchedulerSchemaRunfor):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class WorkflowCronOptionsSchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'description': 'str',
        'schedule': 'str',
        'concurrency_policy': 'str',
        'starting_deadline_duration': 'str',
        'successful_jobs_history_limit': 'int',
        'failed_jobs_history_limit': 'int'
    }

    attribute_map = {
        'description': 'description',
        'schedule': 'schedule',
        'concurrency_policy': 'concurrency-policy',
        'starting_deadline_duration': 'starting-deadline-duration',
        'successful_jobs_history_limit': 'successful-jobs-history-limit',
        'failed_jobs_history_limit': 'failed-jobs-history-limit'
    }

    def __init__(self, description=None, schedule=None, concurrency_policy=None, starting_deadline_duration=None, successful_jobs_history_limit=None, failed_jobs_history_limit=None):  # noqa: E501
        """WorkflowCronOptionsSchema - a model defined in Swagger"""  # noqa: E501

        self._description = None
        self._schedule = None
        self._concurrency_policy = None
        self._starting_deadline_duration = None
        self._successful_jobs_history_limit = None
        self._failed_jobs_history_limit = None
        self.discriminator = None

        if description is not None:
            self.description = description
        self.schedule = schedule
        if concurrency_policy is not None:
            self.concurrency_policy = concurrency_policy
        if starting_deadline_duration is not None:
            self.starting_deadline_duration = starting_deadline_duration
        if successful_jobs_history_limit is not None:
            self.successful_jobs_history_limit = successful_jobs_history_limit
        if failed_jobs_history_limit is not None:
            self.failed_jobs_history_limit = failed_jobs_history_limit

    @property
    def description(self):
        """Gets the description of this WorkflowCronOptionsSchema.  # noqa: E501

        Description about this cron workflow options  # noqa: E501

        :return: The description of this WorkflowCronOptionsSchema.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this WorkflowCronOptionsSchema.

        Description about this cron workflow options  # noqa: E501

        :param description: The description of this WorkflowCronOptionsSchema.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def schedule(self):
        """Gets the schedule of this WorkflowCronOptionsSchema.  # noqa: E501

        Cron expression of time at which workflow will be run  # noqa: E501

        :return: The schedule of this WorkflowCronOptionsSchema.  # noqa: E501
        :rtype: str
        """
        return self._schedule

    @schedule.setter
    def schedule(self, schedule):
        """Sets the schedule of this WorkflowCronOptionsSchema.

        Cron expression of time at which workflow will be run  # noqa: E501

        :param schedule: The schedule of this WorkflowCronOptionsSchema.  # noqa: E501
        :type: str
        """
        if schedule is None:
            raise ValueError("Invalid value for `schedule`, must not be `None`")  # noqa: E501

        self._schedule = schedule

    @property
    def concurrency_policy(self):
        """Gets the concurrency_policy of this WorkflowCronOptionsSchema.  # noqa: E501

        Policy that determines what to do if multiple Workflows are scheduled at the same time  # noqa: E501

        :return: The concurrency_policy of this WorkflowCronOptionsSchema.  # noqa: E501
        :rtype: str
        """
        return self._concurrency_policy

    @concurrency_policy.setter
    def concurrency_policy(self, concurrency_policy):
        """Sets the concurrency_policy of this WorkflowCronOptionsSchema.

        Policy that determines what to do if multiple Workflows are scheduled at the same time  # noqa: E501

        :param concurrency_policy: The concurrency_policy of this WorkflowCronOptionsSchema.  # noqa: E501
        :type: str
        """
        allowed_values = ["allow", "replace", "forbid"]  # noqa: E501
        if concurrency_policy not in allowed_values:
            raise ValueError(
                "Invalid value for `concurrency_policy` ({0}), must be one of {1}"  # noqa: E501
                .format(concurrency_policy, allowed_values)
            )

        self._concurrency_policy = concurrency_policy

    @property
    def starting_deadline_duration(self):
        """Gets the starting_deadline_duration of this WorkflowCronOptionsSchema.  # noqa: E501

        Duration after the last successful run during which a missed Workflow will be run  # noqa: E501

        :return: The starting_deadline_duration of this WorkflowCronOptionsSchema.  # noqa: E501
        :rtype: str
        """
        return self._starting_deadline_duration

    @starting_deadline_duration.setter
    def starting_deadline_duration(self, starting_deadline_duration):
        """Sets the starting_deadline_duration of this WorkflowCronOptionsSchema.

        Duration after the last successful run during which a missed Workflow will be run  # noqa: E501

        :param starting_deadline_duration: The starting_deadline_duration of this WorkflowCronOptionsSchema.  # noqa: E501
        :type: str
        """

        self._starting_deadline_duration = starting_deadline_duration

    @property
    def successful_jobs_history_limit(self):
        """Gets the successful_jobs_history_limit of this WorkflowCronOptionsSchema.  # noqa: E501

        Number of successful Workflows that will be persisted at a time  # noqa: E501

        :return: The successful_jobs_history_limit of this WorkflowCronOptionsSchema.  # noqa: E501
        :rtype: int
        """
        return self._successful_jobs_history_limit

    @successful_jobs_history_limit.setter
    def successful_jobs_history_limit(self, successful_jobs_history_limit):
        """Sets the successful_jobs_history_limit of this WorkflowCronOptionsSchema.

        Number of successful Workflows that will be persisted at a time  # noqa: E501

        :param successful_jobs_history_limit: The successful_jobs_history_limit of this WorkflowCronOptionsSchema.  # noqa: E501
        :type: int
        """

        self._successful_jobs_history_limit = successful_jobs_history_limit

    @property
    def failed_jobs_history_limit(self):
        """Gets the failed_jobs_history_limit of this WorkflowCronOptionsSchema.  # noqa: E501

        Policy that determines what to do if multiple Workflows are scheduled at the same time  # noqa: E501

        :return: The failed_jobs_history_limit of this WorkflowCronOptionsSchema.  # noqa: E501
        :rtype: int
        """
        return self._failed_jobs_history_limit

    @failed_jobs_history_limit.setter
    def failed_jobs_history_limit(self, failed_jobs_history_limit):
        """Sets the failed_jobs_history_limit of this WorkflowCronOptionsSchema.

        Policy that determines what to do if multiple Workflows are scheduled at the same time  # noqa: E501

        :param failed_jobs_history_limit: The failed_jobs_history_limit of this WorkflowCronOptionsSchema.  # noqa: E501
        :type: int
        """

        self._failed_jobs_history_limit = failed_jobs_history_limit

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(WorkflowCronOptionsSchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, WorkflowCronOptionsSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class LicenseKeySchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'license_id': 'str',
        'start_date': 'datetime',
        'end_date': 'datetime',
        'validity_type': 'str',
        'version': 'int',
        'sku_name': 'str',
        'customer_id': 'str',
        'order_type': 'str',
        'sw_serial_id': 'str',
        'mode': 'str',
        'features': 'list[LicensekeySchemaFeatures]'
    }

    attribute_map = {
        'license_id': 'license-id',
        'start_date': 'start-date',
        'end_date': 'end-date',
        'validity_type': 'validity-type',
        'version': 'version',
        'sku_name': 'sku-name',
        'customer_id': 'customer-id',
        'order_type': 'order-type',
        'sw_serial_id': 'sw-serial-id',
        'mode': 'mode',
        'features': 'features'
    }

    def __init__(self, license_id=None, start_date=None, end_date=None, validity_type=None, version=None, sku_name=None, customer_id=None, order_type=None, sw_serial_id=None, mode=None, features=None):  # noqa: E501
        """LicenseKeySchema - a model defined in Swagger"""  # noqa: E501

        self._license_id = None
        self._start_date = None
        self._end_date = None
        self._validity_type = None
        self._version = None
        self._sku_name = None
        self._customer_id = None
        self._order_type = None
        self._sw_serial_id = None
        self._mode = None
        self._features = None
        self.discriminator = None

        self.license_id = license_id
        self.start_date = start_date
        self.end_date = end_date
        self.validity_type = validity_type
        self.version = version
        self.sku_name = sku_name
        self.customer_id = customer_id
        self.order_type = order_type
        if sw_serial_id is not None:
            self.sw_serial_id = sw_serial_id
        if mode is not None:
            self.mode = mode
        self.features = features

    @property
    def license_id(self):
        """Gets the license_id of this LicenseKeySchema.  # noqa: E501

        Unique ID of the license  # noqa: E501

        :return: The license_id of this LicenseKeySchema.  # noqa: E501
        :rtype: str
        """
        return self._license_id

    @license_id.setter
    def license_id(self, license_id):
        """Sets the license_id of this LicenseKeySchema.

        Unique ID of the license  # noqa: E501

        :param license_id: The license_id of this LicenseKeySchema.  # noqa: E501
        :type: str
        """
        if license_id is None:
            raise ValueError("Invalid value for `license_id`, must not be `None`")  # noqa: E501

        self._license_id = license_id

    @property
    def start_date(self):
        """Gets the start_date of this LicenseKeySchema.  # noqa: E501

        License start date and time  # noqa: E501

        :return: The start_date of this LicenseKeySchema.  # noqa: E501
        :rtype: datetime
        """
        return self._start_date

    @start_date.setter
    def start_date(self, start_date):
        """Sets the start_date of this LicenseKeySchema.

        License start date and time  # noqa: E501

        :param start_date: The start_date of this LicenseKeySchema.  # noqa: E501
        :type: datetime
        """
        if start_date is None:
            raise ValueError("Invalid value for `start_date`, must not be `None`")  # noqa: E501

        self._start_date = start_date

    @property
    def end_date(self):
        """Gets the end_date of this LicenseKeySchema.  # noqa: E501

        License end date and time  # noqa: E501

        :return: The end_date of this LicenseKeySchema.  # noqa: E501
        :rtype: datetime
        """
        return self._end_date

    @end_date.setter
    def end_date(self, end_date):
        """Sets the end_date of this LicenseKeySchema.

        License end date and time  # noqa: E501

        :param end_date: The end_date of this LicenseKeySchema.  # noqa: E501
        :type: datetime
        """
        if end_date is None:
            raise ValueError("Invalid value for `end_date`, must not be `None`")  # noqa: E501

        self._end_date = end_date

    @property
    def validity_type(self):
        """Gets the validity_type of this LicenseKeySchema.  # noqa: E501

        License validity type  # noqa: E501

        :return: The validity_type of this LicenseKeySchema.  # noqa: E501
        :rtype: str
        """
        return self._validity_type

    @validity_type.setter
    def validity_type(self, validity_type):
        """Sets the validity_type of this LicenseKeySchema.

        License validity type  # noqa: E501

        :param validity_type: The validity_type of this LicenseKeySchema.  # noqa: E501
        :type: str
        """
        if validity_type is None:
            raise ValueError("Invalid value for `validity_type`, must not be `None`")  # noqa: E501
        allowed_values = ["invalid", "countdown", "date-based", "permanent"]  # noqa: E501
        if validity_type not in allowed_values:
            raise ValueError(
                "Invalid value for `validity_type` ({0}), must be one of {1}"  # noqa: E501
                .format(validity_type, allowed_values)
            )

        self._validity_type = validity_type

    @property
    def version(self):
        """Gets the version of this LicenseKeySchema.  # noqa: E501

        License key version, an integer value indicating version of license vendor info  # noqa: E501

        :return: The version of this LicenseKeySchema.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this LicenseKeySchema.

        License key version, an integer value indicating version of license vendor info  # noqa: E501

        :param version: The version of this LicenseKeySchema.  # noqa: E501
        :type: int
        """
        if version is None:
            raise ValueError("Invalid value for `version`, must not be `None`")  # noqa: E501
        if version is not None and version < 1:  # noqa: E501
            raise ValueError("Invalid value for `version`, must be a value greater than or equal to `1`")  # noqa: E501

        self._version = version

    @property
    def sku_name(self):
        """Gets the sku_name of this LicenseKeySchema.  # noqa: E501

        License stock keeping unit name, indicates category of purchased license  # noqa: E501

        :return: The sku_name of this LicenseKeySchema.  # noqa: E501
        :rtype: str
        """
        return self._sku_name

    @sku_name.setter
    def sku_name(self, sku_name):
        """Sets the sku_name of this LicenseKeySchema.

        License stock keeping unit name, indicates category of purchased license  # noqa: E501

        :param sku_name: The sku_name of this LicenseKeySchema.  # noqa: E501
        :type: str
        """
        if sku_name is None:
            raise ValueError("Invalid value for `sku_name`, must not be `None`")  # noqa: E501

        self._sku_name = sku_name

    @property
    def customer_id(self):
        """Gets the customer_id of this LicenseKeySchema.  # noqa: E501

        Identification of customer who has purchased this license  # noqa: E501

        :return: The customer_id of this LicenseKeySchema.  # noqa: E501
        :rtype: str
        """
        return self._customer_id

    @customer_id.setter
    def customer_id(self, customer_id):
        """Sets the customer_id of this LicenseKeySchema.

        Identification of customer who has purchased this license  # noqa: E501

        :param customer_id: The customer_id of this LicenseKeySchema.  # noqa: E501
        :type: str
        """
        if customer_id is None:
            raise ValueError("Invalid value for `customer_id`, must not be `None`")  # noqa: E501

        self._customer_id = customer_id

    @property
    def order_type(self):
        """Gets the order_type of this LicenseKeySchema.  # noqa: E501

        License purchase order type  # noqa: E501

        :return: The order_type of this LicenseKeySchema.  # noqa: E501
        :rtype: str
        """
        return self._order_type

    @order_type.setter
    def order_type(self, order_type):
        """Sets the order_type of this LicenseKeySchema.

        License purchase order type  # noqa: E501

        :param order_type: The order_type of this LicenseKeySchema.  # noqa: E501
        :type: str
        """
        if order_type is None:
            raise ValueError("Invalid value for `order_type`, must not be `None`")  # noqa: E501
        allowed_values = ["unknown", "commercial", "trial", "demo", "emergency", "lab", "education"]  # noqa: E501
        if order_type not in allowed_values:
            raise ValueError(
                "Invalid value for `order_type` ({0}), must be one of {1}"  # noqa: E501
                .format(order_type, allowed_values)
            )

        self._order_type = order_type

    @property
    def sw_serial_id(self):
        """Gets the sw_serial_id of this LicenseKeySchema.  # noqa: E501

        Software serial number used for license activation  # noqa: E501

        :return: The sw_serial_id of this LicenseKeySchema.  # noqa: E501
        :rtype: str
        """
        return self._sw_serial_id

    @sw_serial_id.setter
    def sw_serial_id(self, sw_serial_id):
        """Sets the sw_serial_id of this LicenseKeySchema.

        Software serial number used for license activation  # noqa: E501

        :param sw_serial_id: The sw_serial_id of this LicenseKeySchema.  # noqa: E501
        :type: str
        """

        self._sw_serial_id = sw_serial_id

    @property
    def mode(self):
        """Gets the mode of this LicenseKeySchema.  # noqa: E501

        License mode of operation  # noqa: E501

        :return: The mode of this LicenseKeySchema.  # noqa: E501
        :rtype: str
        """
        return self._mode

    @mode.setter
    def mode(self, mode):
        """Sets the mode of this LicenseKeySchema.

        License mode of operation  # noqa: E501

        :param mode: The mode of this LicenseKeySchema.  # noqa: E501
        :type: str
        """
        allowed_values = ["invalid", "standalone", "network"]  # noqa: E501
        if mode not in allowed_values:
            raise ValueError(
                "Invalid value for `mode` ({0}), must be one of {1}"  # noqa: E501
                .format(mode, allowed_values)
            )

        self._mode = mode

    @property
    def features(self):
        """Gets the features of this LicenseKeySchema.  # noqa: E501

        Features which are part of the license  # noqa: E501

        :return: The features of this LicenseKeySchema.  # noqa: E501
        :rtype: list[LicensekeySchemaFeatures]
        """
        return self._features

    @features.setter
    def features(self, features):
        """Sets the features of this LicenseKeySchema.

        Features which are part of the license  # noqa: E501

        :param features: The features of this LicenseKeySchema.  # noqa: E501
        :type: list[LicensekeySchemaFeatures]
        """
        if features is None:
            raise ValueError("Invalid value for `features`, must not be `None`")  # noqa: E501

        self._features = features

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(LicenseKeySchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LicenseKeySchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class LicensekeySchemaFeatures(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'feature_id': 'int',
        'feature_name': 'str',
        'feature_description': 'str',
        'capacity_value': 'int',
        'capacity_flag': 'bool'
    }

    attribute_map = {
        'feature_id': 'feature-id',
        'feature_name': 'feature-name',
        'feature_description': 'feature-description',
        'capacity_value': 'capacity-value',
        'capacity_flag': 'capacity-flag'
    }

    def __init__(self, feature_id=None, feature_name=None, feature_description=None, capacity_value=None, capacity_flag=None):  # noqa: E501
        """LicensekeySchemaFeatures - a model defined in Swagger"""  # noqa: E501

        self._feature_id = None
        self._feature_name = None
        self._feature_description = None
        self._capacity_value = None
        self._capacity_flag = None
        self.discriminator = None

        self.feature_id = feature_id
        self.feature_name = feature_name
        self.feature_description = feature_description
        self.capacity_value = capacity_value
        self.capacity_flag = capacity_flag

    @property
    def feature_id(self):
        """Gets the feature_id of this LicensekeySchemaFeatures.  # noqa: E501

        Unique ID of the licensed feature  # noqa: E501

        :return: The feature_id of this LicensekeySchemaFeatures.  # noqa: E501
        :rtype: int
        """
        return self._feature_id

    @feature_id.setter
    def feature_id(self, feature_id):
        """Sets the feature_id of this LicensekeySchemaFeatures.

        Unique ID of the licensed feature  # noqa: E501

        :param feature_id: The feature_id of this LicensekeySchemaFeatures.  # noqa: E501
        :type: int
        """
        if feature_id is None:
            raise ValueError("Invalid value for `feature_id`, must not be `None`")  # noqa: E501
        if feature_id is not None and feature_id < 0:  # noqa: E501
            raise ValueError("Invalid value for `feature_id`, must be a value greater than or equal to `0`")  # noqa: E501

        self._feature_id = feature_id

    @property
    def feature_name(self):
        """Gets the feature_name of this LicensekeySchemaFeatures.  # noqa: E501

        Name of the licensed feature  # noqa: E501

        :return: The feature_name of this LicensekeySchemaFeatures.  # noqa: E501
        :rtype: str
        """
        return self._feature_name

    @feature_name.setter
    def feature_name(self, feature_name):
        """Sets the feature_name of this LicensekeySchemaFeatures.

        Name of the licensed feature  # noqa: E501

        :param feature_name: The feature_name of this LicensekeySchemaFeatures.  # noqa: E501
        :type: str
        """
        if feature_name is None:
            raise ValueError("Invalid value for `feature_name`, must not be `None`")  # noqa: E501

        self._feature_name = feature_name

    @property
    def feature_description(self):
        """Gets the feature_description of this LicensekeySchemaFeatures.  # noqa: E501

        Brief description of the licensed feature  # noqa: E501

        :return: The feature_description of this LicensekeySchemaFeatures.  # noqa: E501
        :rtype: str
        """
        return self._feature_description

    @feature_description.setter
    def feature_description(self, feature_description):
        """Sets the feature_description of this LicensekeySchemaFeatures.

        Brief description of the licensed feature  # noqa: E501

        :param feature_description: The feature_description of this LicensekeySchemaFeatures.  # noqa: E501
        :type: str
        """
        if feature_description is None:
            raise ValueError("Invalid value for `feature_description`, must not be `None`")  # noqa: E501

        self._feature_description = feature_description

    @property
    def capacity_value(self):
        """Gets the capacity_value of this LicensekeySchemaFeatures.  # noqa: E501

        Total capacity of the licensed feature  # noqa: E501

        :return: The capacity_value of this LicensekeySchemaFeatures.  # noqa: E501
        :rtype: int
        """
        return self._capacity_value

    @capacity_value.setter
    def capacity_value(self, capacity_value):
        """Sets the capacity_value of this LicensekeySchemaFeatures.

        Total capacity of the licensed feature  # noqa: E501

        :param capacity_value: The capacity_value of this LicensekeySchemaFeatures.  # noqa: E501
        :type: int
        """
        if capacity_value is None:
            raise ValueError("Invalid value for `capacity_value`, must not be `None`")  # noqa: E501
        if capacity_value is not None and capacity_value < 0:  # noqa: E501
            raise ValueError("Invalid value for `capacity_value`, must be a value greater than or equal to `0`")  # noqa: E501

        self._capacity_value = capacity_value

    @property
    def capacity_flag(self):
        """Gets the capacity_flag of this LicensekeySchemaFeatures.  # noqa: E501

        Flag indicating if the feature is capacity or non-capacity type  # noqa: E501

        :return: The capacity_flag of this LicensekeySchemaFeatures.  # noqa: E501
        :rtype: bool
        """
        return self._capacity_flag

    @capacity_flag.setter
    def capacity_flag(self, capacity_flag):
        """Sets the capacity_flag of this LicensekeySchemaFeatures.

        Flag indicating if the feature is capacity or non-capacity type  # noqa: E501

        :param capacity_flag: The capacity_flag of this LicensekeySchemaFeatures.  # noqa: E501
        :type: bool
        """
        if capacity_flag is None:
            raise ValueError("Invalid value for `capacity_flag`, must not be `None`")  # noqa: E501

        self._capacity_flag = capacity_flag

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(LicensekeySchemaFeatures, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LicensekeySchemaFeatures):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class TsdbPostBodyItems(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'query_name': 'str',
        'device_group': 'str',
        'device': 'str',
        'db': 'str',
        'topic': 'str',
        'rule': 'str',
        'trigger': 'str',
        'measurement': 'str',
        'where': 'str',
        'order': 'str',
        'limit': 'int',
        'fields': 'list[str]',
        'group_by': 'list[str]',
        'outer_queries': 'list[TsdbPostBodyItemsOuterQueries]',
        'device_aggregation': 'TsdbPostBodyItemsDeviceAggregation'
    }

    attribute_map = {
        'query_name': 'queryName',
        'device_group': 'deviceGroup',
        'device': 'device',
        'db': 'db',
        'topic': 'topic',
        'rule': 'rule',
        'trigger': 'trigger',
        'measurement': 'measurement',
        'where': 'where',
        'order': 'order',
        'limit': 'limit',
        'fields': 'fields',
        'group_by': 'GroupBy',
        'outer_queries': 'outerQueries',
        'device_aggregation': 'deviceAggregation'
    }

    def __init__(self, query_name=None, device_group=None, device=None, db=None, topic=None, rule=None, trigger=None, measurement=None, where=None, order=None, limit=None, fields=None, group_by=None, outer_queries=None, device_aggregation=None):  # noqa: E501
        """TsdbPostBodyItems - a model defined in Swagger"""  # noqa: E501

        self._query_name = None
        self._device_group = None
        self._device = None
        self._db = None
        self._topic = None
        self._rule = None
        self._trigger = None
        self._measurement = None
        self._where = None
        self._order = None
        self._limit = None
        self._fields = None
        self._group_by = None
        self._outer_queries = None
        self._device_aggregation = None
        self.discriminator = None

        if query_name is not None:
            self.query_name = query_name
        if device_group is not None:
            self.device_group = device_group
        if device is not None:
            self.device = device
        if db is not None:
            self.db = db
        if topic is not None:
            self.topic = topic
        if rule is not None:
            self.rule = rule
        if trigger is not None:
            self.trigger = trigger
        if measurement is not None:
            self.measurement = measurement
        if where is not None:
            self.where = where
        if order is not None:
            self.order = order
        if limit is not None:
            self.limit = limit
        if fields is not None:
            self.fields = fields
        if group_by is not None:
            self.group_by = group_by
        if outer_queries is not None:
            self.outer_queries = outer_queries
        if device_aggregation is not None:
            self.device_aggregation = device_aggregation

    @property
    def query_name(self):
        """Gets the query_name of this TsdbPostBodyItems.  # noqa: E501

        Name of the query object. Optional. Not used for now  # noqa: E501

        :return: The query_name of this TsdbPostBodyItems.  # noqa: E501
        :rtype: str
        """
        return self._query_name

    @query_name.setter
    def query_name(self, query_name):
        """Sets the query_name of this TsdbPostBodyItems.

        Name of the query object. Optional. Not used for now  # noqa: E501

        :param query_name: The query_name of this TsdbPostBodyItems.  # noqa: E501
        :type: str
        """

        self._query_name = query_name

    @property
    def device_group(self):
        """Gets the device_group of this TsdbPostBodyItems.  # noqa: E501

        Name of the deviceGroup(s). Multiple device groups should be separated by ','. This can be used in combination with device, but is not mandatory. If device is given, then query will be executed only for that particular devices in the given device group, else all devices in group will be considered. Given devices will be applicable for all give device-groups.  # noqa: E501

        :return: The device_group of this TsdbPostBodyItems.  # noqa: E501
        :rtype: str
        """
        return self._device_group

    @device_group.setter
    def device_group(self, device_group):
        """Sets the device_group of this TsdbPostBodyItems.

        Name of the deviceGroup(s). Multiple device groups should be separated by ','. This can be used in combination with device, but is not mandatory. If device is given, then query will be executed only for that particular devices in the given device group, else all devices in group will be considered. Given devices will be applicable for all give device-groups.  # noqa: E501

        :param device_group: The device_group of this TsdbPostBodyItems.  # noqa: E501
        :type: str
        """

        self._device_group = device_group

    @property
    def device(self):
        """Gets the device of this TsdbPostBodyItems.  # noqa: E501

        Name of the device. Multiple device should be separated by ','. This should be used along with deviceGroup. Without deviceGroup, this config will not be considered  # noqa: E501

        :return: The device of this TsdbPostBodyItems.  # noqa: E501
        :rtype: str
        """
        return self._device

    @device.setter
    def device(self, device):
        """Sets the device of this TsdbPostBodyItems.

        Name of the device. Multiple device should be separated by ','. This should be used along with deviceGroup. Without deviceGroup, this config will not be considered  # noqa: E501

        :param device: The device of this TsdbPostBodyItems.  # noqa: E501
        :type: str
        """

        self._device = device

    @property
    def db(self):
        """Gets the db of this TsdbPostBodyItems.  # noqa: E501

        Name of the database. Multiple databases should be separated by ','. '*' can be used to specify all databases.  # noqa: E501

        :return: The db of this TsdbPostBodyItems.  # noqa: E501
        :rtype: str
        """
        return self._db

    @db.setter
    def db(self, db):
        """Sets the db of this TsdbPostBodyItems.

        Name of the database. Multiple databases should be separated by ','. '*' can be used to specify all databases.  # noqa: E501

        :param db: The db of this TsdbPostBodyItems.  # noqa: E501
        :type: str
        """

        self._db = db

    @property
    def topic(self):
        """Gets the topic of this TsdbPostBodyItems.  # noqa: E501

        Name of Healthbot topic. Optional if measurement is used  # noqa: E501

        :return: The topic of this TsdbPostBodyItems.  # noqa: E501
        :rtype: str
        """
        return self._topic

    @topic.setter
    def topic(self, topic):
        """Sets the topic of this TsdbPostBodyItems.

        Name of Healthbot topic. Optional if measurement is used  # noqa: E501

        :param topic: The topic of this TsdbPostBodyItems.  # noqa: E501
        :type: str
        """

        self._topic = topic

    @property
    def rule(self):
        """Gets the rule of this TsdbPostBodyItems.  # noqa: E501

        Name of Healthbot rule. Required if topic is used. Optional if measurement is used  # noqa: E501

        :return: The rule of this TsdbPostBodyItems.  # noqa: E501
        :rtype: str
        """
        return self._rule

    @rule.setter
    def rule(self, rule):
        """Sets the rule of this TsdbPostBodyItems.

        Name of Healthbot rule. Required if topic is used. Optional if measurement is used  # noqa: E501

        :param rule: The rule of this TsdbPostBodyItems.  # noqa: E501
        :type: str
        """

        self._rule = rule

    @property
    def trigger(self):
        """Gets the trigger of this TsdbPostBodyItems.  # noqa: E501

        Name of Healthbot trigger. Optional if measurement is used or rule table is being queried  # noqa: E501

        :return: The trigger of this TsdbPostBodyItems.  # noqa: E501
        :rtype: str
        """
        return self._trigger

    @trigger.setter
    def trigger(self, trigger):
        """Sets the trigger of this TsdbPostBodyItems.

        Name of Healthbot trigger. Optional if measurement is used or rule table is being queried  # noqa: E501

        :param trigger: The trigger of this TsdbPostBodyItems.  # noqa: E501
        :type: str
        """

        self._trigger = trigger

    @property
    def measurement(self):
        """Gets the measurement of this TsdbPostBodyItems.  # noqa: E501

        Name of the measurement. Optional if topic/rule/trigger is used  # noqa: E501

        :return: The measurement of this TsdbPostBodyItems.  # noqa: E501
        :rtype: str
        """
        return self._measurement

    @measurement.setter
    def measurement(self, measurement):
        """Sets the measurement of this TsdbPostBodyItems.

        Name of the measurement. Optional if topic/rule/trigger is used  # noqa: E501

        :param measurement: The measurement of this TsdbPostBodyItems.  # noqa: E501
        :type: str
        """

        self._measurement = measurement

    @property
    def where(self):
        """Gets the where of this TsdbPostBodyItems.  # noqa: E501

        Where clause filters data based on fields, tags, and/or timestamps. Eg: where=\"interface-name\" = 'ge-0/0/1' and \"in-pkts\" > 0  # noqa: E501

        :return: The where of this TsdbPostBodyItems.  # noqa: E501
        :rtype: str
        """
        return self._where

    @where.setter
    def where(self, where):
        """Sets the where of this TsdbPostBodyItems.

        Where clause filters data based on fields, tags, and/or timestamps. Eg: where=\"interface-name\" = 'ge-0/0/1' and \"in-pkts\" > 0  # noqa: E501

        :param where: The where of this TsdbPostBodyItems.  # noqa: E501
        :type: str
        """

        self._where = where

    @property
    def order(self):
        """Gets the order of this TsdbPostBodyItems.  # noqa: E501

        Sort points in descending order based on time. By default points will be sorted in ascending order. Eg: order=desc  # noqa: E501

        :return: The order of this TsdbPostBodyItems.  # noqa: E501
        :rtype: str
        """
        return self._order

    @order.setter
    def order(self, order):
        """Sets the order of this TsdbPostBodyItems.

        Sort points in descending order based on time. By default points will be sorted in ascending order. Eg: order=desc  # noqa: E501

        :param order: The order of this TsdbPostBodyItems.  # noqa: E501
        :type: str
        """

        self._order = order

    @property
    def limit(self):
        """Gets the limit of this TsdbPostBodyItems.  # noqa: E501

        Limit number of points in the result. If groupBy is used limit is applied per group. Eg: limit=10  # noqa: E501

        :return: The limit of this TsdbPostBodyItems.  # noqa: E501
        :rtype: int
        """
        return self._limit

    @limit.setter
    def limit(self, limit):
        """Sets the limit of this TsdbPostBodyItems.

        Limit number of points in the result. If groupBy is used limit is applied per group. Eg: limit=10  # noqa: E501

        :param limit: The limit of this TsdbPostBodyItems.  # noqa: E501
        :type: int
        """

        self._limit = limit

    @property
    def fields(self):
        """Gets the fields of this TsdbPostBodyItems.  # noqa: E501


        :return: The fields of this TsdbPostBodyItems.  # noqa: E501
        :rtype: list[str]
        """
        return self._fields

    @fields.setter
    def fields(self, fields):
        """Sets the fields of this TsdbPostBodyItems.


        :param fields: The fields of this TsdbPostBodyItems.  # noqa: E501
        :type: list[str]
        """

        self._fields = fields

    @property
    def group_by(self):
        """Gets the group_by of this TsdbPostBodyItems.  # noqa: E501


        :return: The group_by of this TsdbPostBodyItems.  # noqa: E501
        :rtype: list[str]
        """
        return self._group_by

    @group_by.setter
    def group_by(self, group_by):
        """Sets the group_by of this TsdbPostBodyItems.


        :param group_by: The group_by of this TsdbPostBodyItems.  # noqa: E501
        :type: list[str]
        """

        self._group_by = group_by

    @property
    def outer_queries(self):
        """Gets the outer_queries of this TsdbPostBodyItems.  # noqa: E501


        :return: The outer_queries of this TsdbPostBodyItems.  # noqa: E501
        :rtype: list[TsdbPostBodyItemsOuterQueries]
        """
        return self._outer_queries

    @outer_queries.setter
    def outer_queries(self, outer_queries):
        """Sets the outer_queries of this TsdbPostBodyItems.


        :param outer_queries: The outer_queries of this TsdbPostBodyItems.  # noqa: E501
        :type: list[TsdbPostBodyItemsOuterQueries]
        """

        self._outer_queries = outer_queries

    @property
    def device_aggregation(self):
        """Gets the device_aggregation of this TsdbPostBodyItems.  # noqa: E501


        :return: The device_aggregation of this TsdbPostBodyItems.  # noqa: E501
        :rtype: TsdbPostBodyItemsDeviceAggregation
        """
        return self._device_aggregation

    @device_aggregation.setter
    def device_aggregation(self, device_aggregation):
        """Sets the device_aggregation of this TsdbPostBodyItems.


        :param device_aggregation: The device_aggregation of this TsdbPostBodyItems.  # noqa: E501
        :type: TsdbPostBodyItemsDeviceAggregation
        """

        self._device_aggregation = device_aggregation

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(TsdbPostBodyItems, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TsdbPostBodyItems):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class RuleSchemaFormulaValuedifference(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'field_name': 'str',
        'hold_time': 'str',
        'multiplication_factor': 'str',
        'extra_keys': 'list[str]'
    }

    attribute_map = {
        'field_name': 'field-name',
        'hold_time': 'hold-time',
        'multiplication_factor': 'multiplication-factor',
        'extra_keys': 'extra-keys'
    }

    def __init__(self, field_name=None, hold_time=None, multiplication_factor=None, extra_keys=None):  # noqa: E501
        """RuleSchemaFormulaValuedifference - a model defined in Swagger"""  # noqa: E501

        self._field_name = None
        self._hold_time = None
        self._multiplication_factor = None
        self._extra_keys = None
        self.discriminator = None

        self.field_name = field_name
        if hold_time is not None:
            self.hold_time = hold_time
        if multiplication_factor is not None:
            self.multiplication_factor = multiplication_factor
        if extra_keys is not None:
            self.extra_keys = extra_keys

    @property
    def field_name(self):
        """Gets the field_name of this RuleSchemaFormulaValuedifference.  # noqa: E501

        Field name on which delta operation needs to be performed  # noqa: E501

        :return: The field_name of this RuleSchemaFormulaValuedifference.  # noqa: E501
        :rtype: str
        """
        return self._field_name

    @field_name.setter
    def field_name(self, field_name):
        """Sets the field_name of this RuleSchemaFormulaValuedifference.

        Field name on which delta operation needs to be performed  # noqa: E501

        :param field_name: The field_name of this RuleSchemaFormulaValuedifference.  # noqa: E501
        :type: str
        """
        if field_name is None:
            raise ValueError("Invalid value for `field_name`, must not be `None`")  # noqa: E501
        if field_name is not None and not re.search(r'^[$][a-z][a-zA-Z0-9_-]*$', field_name):  # noqa: E501
            raise ValueError(r"Invalid value for `field_name`, must be a follow pattern or equal to `/^[$][a-z][a-zA-Z0-9_-]*$/`")  # noqa: E501

        self._field_name = field_name

    @property
    def hold_time(self):
        """Gets the hold_time of this RuleSchemaFormulaValuedifference.  # noqa: E501

        How long previous value should be stored. Should match the pattern [0-9]*(seconds|minutes|hours|days|weeks|years|offset). Default is 1 day  # noqa: E501

        :return: The hold_time of this RuleSchemaFormulaValuedifference.  # noqa: E501
        :rtype: str
        """
        return self._hold_time

    @hold_time.setter
    def hold_time(self, hold_time):
        """Sets the hold_time of this RuleSchemaFormulaValuedifference.

        How long previous value should be stored. Should match the pattern [0-9]*(seconds|minutes|hours|days|weeks|years|offset). Default is 1 day  # noqa: E501

        :param hold_time: The hold_time of this RuleSchemaFormulaValuedifference.  # noqa: E501
        :type: str
        """
        if hold_time is not None and not re.search(r'[0-9]*(\\.[0-9]+)?(seconds|minutes|hours|days|weeks|years|offset)', hold_time):  # noqa: E501
            raise ValueError(r"Invalid value for `hold_time`, must be a follow pattern or equal to `/[0-9]*(\\.[0-9]+)?(seconds|minutes|hours|days|weeks|years|offset)/`")  # noqa: E501

        self._hold_time = hold_time

    @property
    def multiplication_factor(self):
        """Gets the multiplication_factor of this RuleSchemaFormulaValuedifference.  # noqa: E501

        Value to be multiplied with calculated time. Default is 1.0. Should be IEEE-754 64-bit floating-point numbers  # noqa: E501

        :return: The multiplication_factor of this RuleSchemaFormulaValuedifference.  # noqa: E501
        :rtype: str
        """
        return self._multiplication_factor

    @multiplication_factor.setter
    def multiplication_factor(self, multiplication_factor):
        """Sets the multiplication_factor of this RuleSchemaFormulaValuedifference.

        Value to be multiplied with calculated time. Default is 1.0. Should be IEEE-754 64-bit floating-point numbers  # noqa: E501

        :param multiplication_factor: The multiplication_factor of this RuleSchemaFormulaValuedifference.  # noqa: E501
        :type: str
        """

        self._multiplication_factor = multiplication_factor

    @property
    def extra_keys(self):
        """Gets the extra_keys of this RuleSchemaFormulaValuedifference.  # noqa: E501


        :return: The extra_keys of this RuleSchemaFormulaValuedifference.  # noqa: E501
        :rtype: list[str]
        """
        return self._extra_keys

    @extra_keys.setter
    def extra_keys(self, extra_keys):
        """Sets the extra_keys of this RuleSchemaFormulaValuedifference.


        :param extra_keys: The extra_keys of this RuleSchemaFormulaValuedifference.  # noqa: E501
        :type: list[str]
        """

        self._extra_keys = extra_keys

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RuleSchemaFormulaValuedifference, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RuleSchemaFormulaValuedifference):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Paragon Insights APIs

    API interface for PI application  # noqa: E501

    OpenAPI spec version: 4.0.0
    Contact: healthbot-feedback@juniper.net
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class WorkflowInstanceSchema(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'description': 'str',
        'created_at': 'str',
        'started_at': 'str',
        'finished_at': 'str',
        'status': 'str',
        'message': 'str',
        'devices': 'list[str]',
        'device_groups': 'list[str]',
        'parameters': 'list[WorkflowInstanceSchemaParameters]',
        'argument': 'WorkflowArgumentGroupSchema',
        'cron_options': 'WorkflowCronOptionsSchema',
        'batch': 'int',
        'retry': 'RuleSchemaThenRetry',
        'timeout': 'str',
        'workflow_instance_name': 'str',
        'workflow_name': 'str'
    }

    attribute_map = {
        'description': 'description',
        'created_at': 'created-at',
        'started_at': 'started-at',
        'finished_at': 'finished-at',
        'status': 'status',
        'message': 'message',
        'devices': 'devices',
        'device_groups': 'device-groups',
        'parameters': 'parameters',
        'argument': 'argument',
        'cron_options': 'cron-options',
        'batch': 'batch',
        'retry': 'retry',
        'timeout': 'timeout',
        'workflow_instance_name': 'workflow-instance-name',
        'workflow_name': 'workflow-name'
    }

    def __init__(self, description=None, created_at=None, started_at=None, finished_at=None, status=None, message=None, devices=None, device_groups=None, parameters=None, argument=None, cron_options=None, batch=None, retry=None, timeout=None, workflow_instance_name=None, workflow_name=None):  # noqa: E501
        """WorkflowInstanceSchema - a model defined in Swagger"""  # noqa: E501

        self._description = None
        self._created_at = None
        self._started_at = None
        self._finished_at = None
        self._status = None
        self._message = None
        self._devices = None
        self._device_groups = None
        self._parameters = None
        self._argument = None
        self._cron_options = None
        self._batch = None
        self._retry = None
        self._timeout = None
        self._workflow_instance_name = None
        self._workflow_name = None
        self.discriminator = None

        if description is not None:
            self.description = description
        if created_at is not None:
            self.created_at = created_at
        if started_at is not None:
            self.started_at = started_at
        if finished_at is not None:
            self.finished_at = finished_at
        if status is not None:
            self.status = status
        if message is not None:
            self.message = message
        if devices is not None:
            self.devices = devices
        if device_groups is not None:
            self.device_groups = device_groups
        if parameters is not None:
            self.parameters = parameters
        if argument is not None:
            self.argument = argument
        if cron_options is not None:
            self.cron_options = cron_options
        if batch is not None:
            self.batch = batch
        if retry is not None:
            self.retry = retry
        if timeout is not None:
            self.timeout = timeout
        if workflow_instance_name is not None:
            self.workflow_instance_name = workflow_instance_name
        self.workflow_name = workflow_name

    @property
    def description(self):
        """Gets the description of this WorkflowInstanceSchema.  # noqa: E501

        Description about this workflow instance  # noqa: E501

        :return: The description of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this WorkflowInstanceSchema.

        Description about this workflow instance  # noqa: E501

        :param description: The description of this WorkflowInstanceSchema.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def created_at(self):
        """Gets the created_at of this WorkflowInstanceSchema.  # noqa: E501

        Workflow instance creation time  # noqa: E501

        :return: The created_at of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: str
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this WorkflowInstanceSchema.

        Workflow instance creation time  # noqa: E501

        :param created_at: The created_at of this WorkflowInstanceSchema.  # noqa: E501
        :type: str
        """

        self._created_at = created_at

    @property
    def started_at(self):
        """Gets the started_at of this WorkflowInstanceSchema.  # noqa: E501

        Workflow instance startup time  # noqa: E501

        :return: The started_at of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: str
        """
        return self._started_at

    @started_at.setter
    def started_at(self, started_at):
        """Sets the started_at of this WorkflowInstanceSchema.

        Workflow instance startup time  # noqa: E501

        :param started_at: The started_at of this WorkflowInstanceSchema.  # noqa: E501
        :type: str
        """

        self._started_at = started_at

    @property
    def finished_at(self):
        """Gets the finished_at of this WorkflowInstanceSchema.  # noqa: E501

        Workflow instance completion time  # noqa: E501

        :return: The finished_at of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: str
        """
        return self._finished_at

    @finished_at.setter
    def finished_at(self, finished_at):
        """Sets the finished_at of this WorkflowInstanceSchema.

        Workflow instance completion time  # noqa: E501

        :param finished_at: The finished_at of this WorkflowInstanceSchema.  # noqa: E501
        :type: str
        """

        self._finished_at = finished_at

    @property
    def status(self):
        """Gets the status of this WorkflowInstanceSchema.  # noqa: E501

        Workflow instance current status  # noqa: E501

        :return: The status of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this WorkflowInstanceSchema.

        Workflow instance current status  # noqa: E501

        :param status: The status of this WorkflowInstanceSchema.  # noqa: E501
        :type: str
        """

        self._status = status

    @property
    def message(self):
        """Gets the message of this WorkflowInstanceSchema.  # noqa: E501

        Workflow instance current status message  # noqa: E501

        :return: The message of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this WorkflowInstanceSchema.

        Workflow instance current status message  # noqa: E501

        :param message: The message of this WorkflowInstanceSchema.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def devices(self):
        """Gets the devices of this WorkflowInstanceSchema.  # noqa: E501


        :return: The devices of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._devices

    @devices.setter
    def devices(self, devices):
        """Sets the devices of this WorkflowInstanceSchema.


        :param devices: The devices of this WorkflowInstanceSchema.  # noqa: E501
        :type: list[str]
        """

        self._devices = devices

    @property
    def device_groups(self):
        """Gets the device_groups of this WorkflowInstanceSchema.  # noqa: E501


        :return: The device_groups of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: list[str]
        """
        return self._device_groups

    @device_groups.setter
    def device_groups(self, device_groups):
        """Sets the device_groups of this WorkflowInstanceSchema.


        :param device_groups: The device_groups of this WorkflowInstanceSchema.  # noqa: E501
        :type: list[str]
        """

        self._device_groups = device_groups

    @property
    def parameters(self):
        """Gets the parameters of this WorkflowInstanceSchema.  # noqa: E501


        :return: The parameters of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: list[WorkflowInstanceSchemaParameters]
        """
        return self._parameters

    @parameters.setter
    def parameters(self, parameters):
        """Sets the parameters of this WorkflowInstanceSchema.


        :param parameters: The parameters of this WorkflowInstanceSchema.  # noqa: E501
        :type: list[WorkflowInstanceSchemaParameters]
        """

        self._parameters = parameters

    @property
    def argument(self):
        """Gets the argument of this WorkflowInstanceSchema.  # noqa: E501


        :return: The argument of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: WorkflowArgumentGroupSchema
        """
        return self._argument

    @argument.setter
    def argument(self, argument):
        """Sets the argument of this WorkflowInstanceSchema.


        :param argument: The argument of this WorkflowInstanceSchema.  # noqa: E501
        :type: WorkflowArgumentGroupSchema
        """

        self._argument = argument

    @property
    def cron_options(self):
        """Gets the cron_options of this WorkflowInstanceSchema.  # noqa: E501


        :return: The cron_options of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: WorkflowCronOptionsSchema
        """
        return self._cron_options

    @cron_options.setter
    def cron_options(self, cron_options):
        """Sets the cron_options of this WorkflowInstanceSchema.


        :param cron_options: The cron_options of this WorkflowInstanceSchema.  # noqa: E501
        :type: WorkflowCronOptionsSchema
        """

        self._cron_options = cron_options

    @property
    def batch(self):
        """Gets the batch of this WorkflowInstanceSchema.  # noqa: E501

        Maximum parallel steps launched  # noqa: E501

        :return: The batch of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: int
        """
        return self._batch

    @batch.setter
    def batch(self, batch):
        """Sets the batch of this WorkflowInstanceSchema.

        Maximum parallel steps launched  # noqa: E501

        :param batch: The batch of this WorkflowInstanceSchema.  # noqa: E501
        :type: int
        """

        self._batch = batch

    @property
    def retry(self):
        """Gets the retry of this WorkflowInstanceSchema.  # noqa: E501


        :return: The retry of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: RuleSchemaThenRetry
        """
        return self._retry

    @retry.setter
    def retry(self, retry):
        """Sets the retry of this WorkflowInstanceSchema.


        :param retry: The retry of this WorkflowInstanceSchema.  # noqa: E501
        :type: RuleSchemaThenRetry
        """

        self._retry = retry

    @property
    def timeout(self):
        """Gets the timeout of this WorkflowInstanceSchema.  # noqa: E501

        Maximum time to wait for the step completion before bailing out (default 60 seconds)  # noqa: E501

        :return: The timeout of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: str
        """
        return self._timeout

    @timeout.setter
    def timeout(self, timeout):
        """Sets the timeout of this WorkflowInstanceSchema.

        Maximum time to wait for the step completion before bailing out (default 60 seconds)  # noqa: E501

        :param timeout: The timeout of this WorkflowInstanceSchema.  # noqa: E501
        :type: str
        """

        self._timeout = timeout

    @property
    def workflow_instance_name(self):
        """Gets the workflow_instance_name of this WorkflowInstanceSchema.  # noqa: E501

        Name of the workflow instance  # noqa: E501

        :return: The workflow_instance_name of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: str
        """
        return self._workflow_instance_name

    @workflow_instance_name.setter
    def workflow_instance_name(self, workflow_instance_name):
        """Sets the workflow_instance_name of this WorkflowInstanceSchema.

        Name of the workflow instance  # noqa: E501

        :param workflow_instance_name: The workflow_instance_name of this WorkflowInstanceSchema.  # noqa: E501
        :type: str
        """
        if workflow_instance_name is not None and len(workflow_instance_name) < 1:
            raise ValueError("Invalid value for `workflow_instance_name`, length must be greater than or equal to `1`")  # noqa: E501

        self._workflow_instance_name = workflow_instance_name

    @property
    def workflow_name(self):
        """Gets the workflow_name of this WorkflowInstanceSchema.  # noqa: E501

        Name of the workflow. Should be of pattern [a-zA-Z][a-zA-Z0-9_-]*  # noqa: E501

        :return: The workflow_name of this WorkflowInstanceSchema.  # noqa: E501
        :rtype: str
        """
        return self._workflow_name

    @workflow_name.setter
    def workflow_name(self, workflow_name):
        """Sets the workflow_name of this WorkflowInstanceSchema.

        Name of the workflow. Should be of pattern [a-zA-Z][a-zA-Z0-9_-]*  # noqa: E501

        :param workflow_name: The workflow_name of this WorkflowInstanceSchema.  # noqa: E501
        :type: str
        """
        if workflow_name is None:
            raise ValueError("Invalid value for `workflow_name`, must not be `None`")  # noqa: E501
        if workflow_name is not None and len(workflow_name) > 64:
            raise ValueError("Invalid value for `workflow_name`, length must be less than or equal to `64`")  # noqa: E501
        if workflow_name is not None and len(workflow_name) < 1:
            raise ValueError("Invalid value for `workflow_name`, length must be greater than or equal to `1`")  # noqa: E501
        if workflow_name is not None and not re.search(r'^[a-zA-Z][a-zA-Z0-9_-]*$', workflow_name):  # noqa: E501
            raise ValueError(r"Invalid value for `workflow_name`, must be a follow pattern or equal to `/^[a-zA-Z][a-zA-Z0-9_-]*$/`")  # noqa: E501

        self._workflow_name = workflow_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(WorkflowInstanceSchema, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, WorkflowInstanceSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
